classdef Cycle < handle & dynamicprops
    %CYCLE Class containing driving cycle data and exhaust concentrations
    %in tailpipe, CVS sampling system and downstream OFRs.
    %   Pauli Simonen, Tampere University, 2023
    
    properties (SetAccess=protected, GetAccess=public)
        name
        original % Struct containing original gas data and calculated values:
            % co2_percent
            % hc_ppm
            % speed
            % voc_g_per_m3
            % co2_g_per_m3
            % exhFlow_m3_per_s
            % ....
        ofrNames
        
    end

    properties (Access = protected)
        soaYield
        M_co2
        M_ch4
        N_A
    end
    
    methods
        function obj = Cycle(name, cycleStruct, soaYield, endBufferLength)
            %CYCLE Construct an instance of this class
            %   Detailed explanation goes here

            obj.soaYield = soaYield;
            obj.M_co2 = 44; % g/mol
            obj.N_A = 6.022e23;
            obj.M_ch4 = 16; % g/mol

            M_co2 = obj.M_co2;
            N_A = obj.N_A;
            M_ch4 = obj.M_ch4;

            obj.ofrNames = {};
            
            %% Add original cycle data
            requiredFields = {'time','co2_percent','speed','hc_ppm',...
                'exhFlow_m3_per_s'};
            for k=1:numel(requiredFields)
                if(~isfield(cycleStruct,requiredFields{k}))
                    error(['cycleStruct must contain a field named '''...
                        requiredFields{k} '''.']);
                end
                if(strcmp(requiredFields{k},'time') ||...
                        strcmp(requiredFields{k},'exhFlow_m3_per_s') ||...
                        strcmp(requiredFields{k},'speed'))
                    obj.original.(requiredFields{k}) = cycleStruct.(requiredFields{k});
                else
                    obj.original.tailpipe.(requiredFields{k}) =...
                        cycleStruct.(requiredFields{k});
                end
            end
            
            obj.name = name;

            %% Calculate mass concentrations:
            obj.original.tailpipe.hc_g_per_m3 =...
                ppmTo1Cm3(obj.original.tailpipe.hc_ppm, 30)./N_A.*1e6.*M_ch4;
            obj.original.tailpipe.co2_g_per_m3 =...
                ppmTo1Cm3(obj.original.tailpipe.co2_percent.*1e4,30)./N_A.*1e6.*M_co2;
            
            %% Calculate SOA formation potential ('SOA concentration'):
            obj.original.tailpipe.soa_g_per_m3 = obj.original.tailpipe.hc_g_per_m3.*soaYield;
            
            %% Assign CVS dilution ratio:
            zeroIndices = obj.original.exhFlow_m3_per_s < 1e-6;
            obj.original.cvs.dr = max(obj.original.exhFlow_m3_per_s)./...
                    obj.original.exhFlow_m3_per_s.*10; % CVS dilution ratio
            obj.original.cvs.flow_m3_per_s =...
                max(obj.original.exhFlow_m3_per_s).*10; % CVS total flow
            concFields = {'co2_percent','hc_ppm','co2_g_per_m3',...
                'hc_g_per_m3','soa_g_per_m3'};
            for k = 1:numel(concFields)
                obj.original.cvs.(concFields{k}) =...
                    obj.original.tailpipe.(concFields{k})./obj.original.cvs.dr;
                obj.original.cvs.(concFields{k})(zeroIndices) = 0;
            end

            %% Add end buffer to the cycle
            % To allow simulating OFR concentrations beyond the end of the
            % cycle.

            % Store the original cycle length before extending:
            obj.original.cycleLength = numel(cycleStruct.time);
            
            obj.original.time = [obj.original.time;...
                        obj.original.time(end) + (1:1:endBufferLength)'];
            obj.original.exhFlow_m3_per_s = [obj.original.exhFlow_m3_per_s;...
                zeros(endBufferLength,1)];

            fields = fieldnames(obj.original.tailpipe);
            for k = 1:numel(fields)
                % Assume that the OFRs start sampling zero air right
                % after the cycle ends, so all tailpipe concentrations
                % become zero, and the exhaust flow rate is zero:
                obj.original.tailpipe.(fields{k}) = [obj.original.tailpipe.(fields{k});...
                    zeros(endBufferLength,1)];
            end

            fields = fieldnames(obj.original.cvs);
            for k = 1:numel(fields)
                if(strcmp(fields{k},'dr'))
                    obj.original.cvs.dr = [obj.original.cvs.dr;...
                        zeros(endBufferLength,1)]; % Inf would be more correct here, but it messes up the cumulative emission calculations later.
                elseif(strcmp(fields{k},'flow_m3_per_s'))
                    continue; % This is not a vector.
                else
                    % Assume that the OFRs start sampling zero air right
                    % after the cycle ends, so all tailpipe concentrations
                    % become zero, and the exhaust flow rate is zero:
                    obj.original.cvs.(fields{k}) = [obj.original.cvs.(fields{k});...
                        zeros(endBufferLength,1)];
                end
            end

            %% Calculate cumulative emissions (g) and SOA PF (g/kgfuel):
            % Conversion factor to convert (g/g_co2) to (g/kg_fuel):
            kk = 860*44/12; % 860 g carbon / kg fuel
            
            obj.original.emissions.cumulativeCo2_g =...
                cumsum(obj.original.tailpipe.co2_g_per_m3.*...
                obj.original.exhFlow_m3_per_s);
            obj.original.emissions.cumulativeSoa_g =...
                cumsum(obj.original.tailpipe.soa_g_per_m3.*...
                obj.original.exhFlow_m3_per_s);
                            
            % Cumulative SOA PF; e.g the value cumulativePf(100) tells
            % the total SOA PF between 0...100 s and cumulativePf(end)
            % tells the SOA PF of the full cycle.
            obj.original.emissions.cumulativePf_mg_per_kgfuel =...
                obj.original.emissions.cumulativeSoa_g./...
                obj.original.emissions.cumulativeCo2_g.*kk.*1e3;
            
            % Time-dependent SOA PF; e.g. the value pf(100) tells the
            % momentary ratio between emitted SOA and emitted CO2 at
            % 100 s.
            obj.original.emissions.pf_mg_per_kgfuel =...
                obj.original.tailpipe.soa_g_per_m3./...
                obj.original.tailpipe.co2_g_per_m3.*kk.*1e3;
        end

        function setOfrDelayCorrection(obj, ofrName, delayCorrection_s)
            if(~isprop(obj,ofrName))
                error(['No OFR ''' ofrName ''' found in cycle struct.']);
            end
            obj.(ofrName).delayCorrection_s = delayCorrection_s;

            % Recalculate the emissions:
            calculateOfrData(obj, ofrName);
        end
        
        function calculateOfrData(obj, ofrName)
            if(~isprop(obj,ofrName))
                error(['No OFR ''' ofrName ''' found in cycle struct.']);
            end
            M_co2 = obj.M_co2;
            N_A = obj.N_A;
            M_ch4 = obj.M_ch4;
            soaYield = obj.soaYield;

            %% Simulate OFR outlet concentrations w/ and w/o delay correction:
            obj.(ofrName).exhFlow_m3_per_s =...
                    simulateRtd(obj.original.exhFlow_m3_per_s, obj.(ofrName).E, 0);
            obj.(ofrName).exhFlow_m3_per_s_shifted =...
                    simulateRtd(obj.original.exhFlow_m3_per_s, obj.(ofrName).E,...
                    obj.(ofrName).delayCorrection_s);
            

            samplingPoints = {'tailpipe','cvs'};
            for k=1:numel(samplingPoints)
                obj.(ofrName).(samplingPoints{k}).co2_percent =...
                    simulateRtd(obj.original.(samplingPoints{k}).co2_percent, obj.(ofrName).E, 0);
                obj.(ofrName).(samplingPoints{k}).co2_percent_shifted =...
                    simulateRtd(obj.original.(samplingPoints{k}).co2_percent, obj.(ofrName).E, obj.(ofrName).delayCorrection_s);
                obj.(ofrName).(samplingPoints{k}).hc_ppm =...
                    simulateRtd(obj.original.(samplingPoints{k}).hc_ppm, obj.(ofrName).E, 0);
                obj.(ofrName).(samplingPoints{k}).hc_ppm_shifted =...
                    simulateRtd(obj.original.(samplingPoints{k}).hc_ppm, obj.(ofrName).E, obj.(ofrName).delayCorrection_s);
                

                %% Calculate mass concentrations:    
                obj.(ofrName).(samplingPoints{k}).hc_g_per_m3 =...
                    ppmTo1Cm3(obj.(ofrName).(samplingPoints{k}).hc_ppm, 30)./N_A.*1e6.*M_ch4;
                obj.(ofrName).(samplingPoints{k}).hc_g_per_m3_shifted =...
                    ppmTo1Cm3(obj.(ofrName).(samplingPoints{k}).hc_ppm_shifted, 30)./N_A.*1e6.*M_ch4;
                
                obj.(ofrName).(samplingPoints{k}).co2_g_per_m3 =...
                    ppmTo1Cm3(obj.(ofrName).(samplingPoints{k}).co2_percent.*1e4,30)./N_A.*1e6.*M_co2;
                obj.(ofrName).(samplingPoints{k}).co2_g_per_m3_shifted =...
                    ppmTo1Cm3(obj.(ofrName).(samplingPoints{k}).co2_percent_shifted.*1e4,30)./N_A.*1e6.*M_co2;
                
                %% Calculate SOA formation potential ('SOA concentration'):
                obj.(ofrName).(samplingPoints{k}).soa_g_per_m3 =...
                    obj.(ofrName).(samplingPoints{k}).hc_g_per_m3.*soaYield;
                obj.(ofrName).(samplingPoints{k}).soa_g_per_m3_shifted =...
                    obj.(ofrName).(samplingPoints{k}).hc_g_per_m3_shifted.*soaYield;
            end

            %% Calculate emissions from OFR data with different methods:
            % Conversion factor to convert (g/g_co2) to (g/kg_fuel):
            kk = 860*44/12; % 860 g carbon / kg fuel
            methods = {'standard','OFR CO_2','convolution','CVS','averaging',...
                'deconvolution','CVS_noDelayCorrection'};
            obj.(ofrName).emissions = cell(numel(methods),1);

            % In all methods, the emissions are calculated from
            % delay-corrected data, except for the 'Deconvolution' method,
            % which inherently takes the delay into account. The
            % 'CVS_noDelayCorrection' method also neglects the delay
            % correction - when calculating the full cycle PF, this method
            % reports the correct PF.
            for k = 1:numel(methods)
                obj.(ofrName).emissions{k} = struct;
                obj.(ofrName).emissions{k}.method = methods{k};
                switch(methods{k})
                    case 'standard'
                        % Use original Q_exh and normalize to original CO2.
                        obj.(ofrName).emissions{k}.cumulativeCo2_g =...
                            obj.original.emissions.cumulativeCo2_g;
                        obj.(ofrName).emissions{k}.cumulativeSoa_g =...
                            cumsum(obj.(ofrName).tailpipe.soa_g_per_m3_shifted.*...
                            obj.original.exhFlow_m3_per_s);
                        obj.(ofrName).emissions{k}.cumulativePf_mg_per_kgfuel =...
                            obj.(ofrName).emissions{k}.cumulativeSoa_g./...
                            obj.(ofrName).emissions{k}.cumulativeCo2_g.*kk.*1e3;
                        obj.(ofrName).emissions{k}.pf_mg_per_kgfuel =...
                            obj.(ofrName).tailpipe.soa_g_per_m3_shifted./...
                            obj.original.tailpipe.co2_g_per_m3.*kk.*1e3;
                    case 'OFR CO_2'
                        % Use original Q_exh and calculate the CO2 emission
                        % based on the OFR outlet CO2 concentration
                        obj.(ofrName).emissions{k}.cumulativeCo2_g =...
                            cumsum(obj.(ofrName).tailpipe.co2_g_per_m3_shifted.*...
                            obj.original.exhFlow_m3_per_s);
                        obj.(ofrName).emissions{k}.cumulativeSoa_g =...
                            cumsum(obj.(ofrName).tailpipe.soa_g_per_m3_shifted.*...
                            obj.original.exhFlow_m3_per_s);
                        obj.(ofrName).emissions{k}.cumulativePf_mg_per_kgfuel =...
                            obj.(ofrName).emissions{k}.cumulativeSoa_g./...
                            obj.(ofrName).emissions{k}.cumulativeCo2_g.*kk.*1e3;
                        obj.(ofrName).emissions{k}.pf_mg_per_kgfuel =...
                            obj.(ofrName).tailpipe.soa_g_per_m3_shifted./...
                            obj.(ofrName).tailpipe.co2_g_per_m3_shifted.*kk.*1e3;
                    case 'convolution'
                        % Use Q_exh convoluted with OFR RTD, and the OFR
                        % outlet concentrations of CO2 and SOA
                        obj.(ofrName).emissions{k}.cumulativeCo2_g =...
                            cumsum(obj.(ofrName).tailpipe.co2_g_per_m3_shifted.*...
                            obj.(ofrName).exhFlow_m3_per_s_shifted);
                        obj.(ofrName).emissions{k}.cumulativeSoa_g =...
                            cumsum(obj.(ofrName).tailpipe.soa_g_per_m3_shifted.*...
                            obj.(ofrName).exhFlow_m3_per_s_shifted);
                        obj.(ofrName).emissions{k}.cumulativePf_mg_per_kgfuel =...
                            obj.(ofrName).emissions{k}.cumulativeSoa_g./...
                            obj.(ofrName).emissions{k}.cumulativeCo2_g.*kk.*1e3;
                        obj.(ofrName).emissions{k}.pf_mg_per_kgfuel =...
                            obj.(ofrName).tailpipe.soa_g_per_m3_shifted./...
                            obj.(ofrName).tailpipe.co2_g_per_m3_shifted.*kk.*1e3;
                    case 'CVS'
                        % Use the CVS data and multiply with CVS constant
                        % flow
                        obj.(ofrName).emissions{k}.cumulativeCo2_g =...
                            cumsum(obj.(ofrName).cvs.co2_g_per_m3_shifted.*...
                            obj.original.cvs.flow_m3_per_s);
                        obj.(ofrName).emissions{k}.cumulativeSoa_g =...
                            cumsum(obj.(ofrName).cvs.soa_g_per_m3_shifted.*...
                            obj.original.cvs.flow_m3_per_s);
                        obj.(ofrName).emissions{k}.cumulativePf_mg_per_kgfuel =...
                            obj.(ofrName).emissions{k}.cumulativeSoa_g./...
                            obj.(ofrName).emissions{k}.cumulativeCo2_g.*kk.*1e3;
                        obj.(ofrName).emissions{k}.pf_mg_per_kgfuel =...
                            obj.(ofrName).cvs.soa_g_per_m3_shifted./...
                            obj.(ofrName).cvs.co2_g_per_m3_shifted.*kk.*1e3;
                    case 'averaging'
                        % No cumulative emissions available for the
                        % averaging method, only the momentary SOA PF and
                        % cumulative SOA PF (average of momentary between 0
                        % and x).
                        obj.(ofrName).emissions{k}.cumulativeCo2_g =...
                            NaN(size(obj.original.emissions.cumulativeCo2_g));
                        obj.(ofrName).emissions{k}.cumulativeSoa_g =...
                            NaN(size(obj.original.emissions.cumulativeSoa_g));
                        obj.(ofrName).emissions{k}.pf_mg_per_kgfuel =...
                            obj.(ofrName).tailpipe.soa_g_per_m3_shifted./...
                            obj.(ofrName).tailpipe.co2_g_per_m3_shifted.*kk.*1e3;
                        obj.(ofrName).emissions{k}.cumulativePf_mg_per_kgfuel =...
                            cumsum(obj.(ofrName).emissions{k}.pf_mg_per_kgfuel)./...
                            (1:numel(obj.(ofrName).emissions{k}.pf_mg_per_kgfuel))';
                    case 'deconvolution'
                        if(~isfield(obj.(ofrName),'tailpipeDeconvoluted'))
                            warning('Cycle:deconvolution',...
                                ['No deconvoluted data available, '...
                                'skipping the deconvolution method.']);
                            obj.(ofrName).emissions{k}.cumulativeCo2_g =...
                                NaN(size(obj.original.emissions.cumulativeCo2_g));
                            obj.(ofrName).emissions{k}.cumulativeSoa_g =...
                                NaN(size(obj.original.emissions.cumulativeSoa_g));
                            obj.(ofrName).emissions{k}.cumulativePf_mg_per_kgfuel =...
                                NaN(size(obj.original.emissions.cumulativeSoa_g));
                            obj.(ofrName).emissions{k}.pf_mg_per_kgfuel =...
                                NaN(size(obj.original.emissions.cumulativeSoa_g));
                        else
                            % Use deconvoluted OFR data and normalize to
                            % true CO2 emission.
                            obj.(ofrName).emissions{k}.cumulativeCo2_g =...
                                obj.original.emissions.cumulativeCo2_g;
                            % Deconvolved SOA contains NaN values at the
                            % end (to have the same size as original data).
                            % Omit the NaNs when calculating cumulative
                            % SOA:
                            obj.(ofrName).emissions{k}.cumulativeSoa_g =...
                                cumsum(obj.(ofrName).tailpipeDeconvoluted.soa_g_per_m3.*...
                                obj.original.exhFlow_m3_per_s,'omitnan');
                            obj.(ofrName).emissions{k}.cumulativePf_mg_per_kgfuel =...
                                obj.(ofrName).emissions{k}.cumulativeSoa_g./...
                                obj.(ofrName).emissions{k}.cumulativeCo2_g.*kk.*1e3;
                            obj.(ofrName).emissions{k}.pf_mg_per_kgfuel =...
                                obj.(ofrName).tailpipeDeconvoluted.soa_g_per_m3./...
                                obj.original.tailpipe.co2_g_per_m3.*kk.*1e3;                            
                        end
                    case 'CVS_noDelayCorrection'
                        % Use the CVS data and multiply with CVS constant
                        % flow
                        obj.(ofrName).emissions{k}.cumulativeCo2_g =...
                            cumsum(obj.(ofrName).cvs.co2_g_per_m3.*...
                            obj.original.cvs.flow_m3_per_s);
                        obj.(ofrName).emissions{k}.cumulativeSoa_g =...
                            cumsum(obj.(ofrName).cvs.soa_g_per_m3.*...
                            obj.original.cvs.flow_m3_per_s);
                        obj.(ofrName).emissions{k}.cumulativePf_mg_per_kgfuel =...
                            obj.(ofrName).emissions{k}.cumulativeSoa_g./...
                            obj.(ofrName).emissions{k}.cumulativeCo2_g.*kk.*1e3;
                        obj.(ofrName).emissions{k}.pf_mg_per_kgfuel =...
                            obj.(ofrName).cvs.soa_g_per_m3./...
                            obj.(ofrName).cvs.co2_g_per_m3.*kk.*1e3;
                    otherwise
                        error(['Unknown calculation method: ' methods{k}]);
                end
            end
        end

        function addOfr(obj, ofrClass, varargin)
            % Inputs: varargin = deconvoluted SOA concentration

            ofrName = ofrClass.name;
            
            if(isprop(obj,ofrName))
                while(true)
                    ip = input(['OFR ''' ofrName ''' already exists. Replace (y/n)?'],'s');
                    if(strcmpi(ip,'n'))
                        return;
                    elseif(strcmpi(ip,'y'))
                        break;
                    end
                end
            else
                newProp = obj.addprop(ofrName);
                newProp.SetAccess = 'protected';
                newProp.GetAccess = 'public';
                obj.ofrNames{1,end+1} = ofrName;
            end
            obj.(ofrName).time = obj.original.time;
            % Store the delay correction
            obj.(ofrName).delayCorrection_s = ofrClass.t_peak; % t_peak by default
            % Store the transfer function
            obj.(ofrName).E = ofrClass.rtd;

            if(nargin == 3)
                % Add deconvoluted data:
                
                % The size of deconvoluted data is not
                % necessarily as long as the original extended
                % data:
                len = numel(varargin{1});
                origLen = numel(obj.original.time);
                if(len < origLen)
                    deconvSoa = zeros(size(obj.original.time));
                    deconvSoa(1:len) = varargin{1};
                elseif(len > origLen)
                    deconvSoa = varargin{1}(1:origLen);
                else
                    deconvSoa = varargin{1};
                end
                                
                obj.(ofrName).tailpipeDeconvoluted.soa_g_per_m3 = deconvSoa;
            end

            % Simulate the concentrations downstream of OFR and calculate
            % the emissions:
            calculateOfrData(obj, ofrName);
        end
         
        function deconvolveOfrData(obj, ofrName, rtdLength, maxIterations)
            % Calculates deconvoluted OFR data based on convolved tailpipe
            % concentrations.
            % rtdLength = length of transfer function. The longer the
            % transfer function, the longer it takes to deconvolve. In
            % general, 150 s is sufficient for DOFR, and 500 s is used for
            % PAM because its RTD is broader.

            % Check if OFR exists:
            if(~isprop(obj,ofrName))
                error(['No OFR ''' ofrName ''' found in cycle struct.']);
            end

            startTime = now();
            disp(['Deconvolution for ' ofrName ' started at '...
                datestr(startTime,'yyyy-mm-dd HH:MM') '.']);

            % Normalize the data (maybe not necessary...):
            normalizer = max(obj.(ofrName).tailpipe.soa_g_per_m3);

            % OFR output:
            Cout = obj.(ofrName).tailpipe.soa_g_per_m3./normalizer;
            Cout = Cout';

            E = obj.(ofrName).E(1:rtdLength);

            % Re-normalize the shortened transfer function:
            E = E./sum(E);

            v = numel(Cout);
            n = numel(E);
            m = v-n+1;

            if(v < n)
                error('Cycle duration is shorter than the length of OFR transfer function. Try increasing the endBufferLength or decreasing the rtdLength');
            end
            
            A = zeros(m+n-1,m);
            
            for k=1:m
                A(k:k+n-1,k) = E;
            end

            Cin0 = zeros(1,m); % The initial guess

            f = @(x)minimizer(obj,x,A,Cout);

            Aa = [];
            b = [];
            Aeq = [];
            beq = [];
            lb = zeros(size(Cin0)); % Lower boundary = 0
            ub = []; % No upper boundary
            nonlcon = [];
            
            opts = optimoptions('fmincon','MaxFunEvals',maxIterations,'MaxIter',maxIterations);

            Cin = fmincon(f,Cin0,Aa,b,Aeq,beq,lb,ub,nonlcon,opts);
            
            % Non-normalize Cin:
            Cin = Cin.*normalizer;
            
            % Deconvolved data will be shorter than original data. Fill
            % missing data with NaNs.
            nans2add = numel(obj.original.time)-numel(Cin);

            obj.(ofrName).tailpipeDeconvoluted.soa_g_per_m3 =...
                [Cin'; NaN(nans2add,1)];


            endTime = now();
            disp(['Deconvolution for ' ofrName ' finished at '...
                datestr(endTime,'yyyy-mm-dd HH:MM') '.']);
            disp(['Total duration: ' num2str((endTime-startTime)*60*24),...
                ' min.']);


            % Recalculate the emissions:
            calculateOfrData(obj, ofrName);
        end

        function OF = minimizer(obj, Cin0, A, Cout)
            % Calculates the square sum of residuals between Cout (actual
            % measurement downstream the OFR) and Cin (the guess of input
            % concentration convolved with the OFR transfer function).
            
            S = A*Cin0'; % Convolved data
            OF = sum((Cout'-S).^2);
        
        end

        function pf_mg_kgfuel = soaPfBetween(obj, startTime, endTime, source, varargin)
            % Calculates SOA PF between timeStart and timeEnd
            % Source is either 'original' or any of OFRs
            % When the source is an OFR, user must input the calculation
            % method as well, e.g.: soaPfBetween(1,100,'DOFR','standard').

            % Conversion factor to convert (g/g_co2) to (g/kg_fuel):
            kk = 860*44/12; % 860 g carbon / kg fuel

            if(~strcmp(source,'original'))
                if(nargin ~= 5)
                    error('Wrong number of arguments, please input the calculation method as well.');
                end
                if(~any(strcmp(obj.ofrNames,source)))
                    error(['Could not find OFR ' source ' in the cycle struct.']);
                end
                method = varargin{1};
                % Find the correct index for the requested method:
                emissionIx = NaN;
                for k=1:numel(obj.(source).emissions)
                    if(strcmp(obj.(source).emissions{k}.method, method))
                        emissionIx = k;
                        break;
                    end
                end
                if(isnan(emissionIx))
                    error(['Could not find calculation method ''' method '''.']);
                end
                if(strcmp(method,'averaging'))
                    % For averaging method, PF = mean(momentaryPf).
                    if(isnan(endTime))
                        % Do not calculate the average beyond the original
                        % end point of cycle:
                        endTime = obj.original.cycleLength;
                    end
                    pf_mg_kgfuel = mean(...
                        obj.(source).emissions{emissionIx}.pf_mg_per_kgfuel(startTime:endTime));
                else
                    % PF = (emittedSoa(end)-emittedSoa(start)) /
                    % (emittedCo2(end)-emittedCo2(start))
                    if(isnan(endTime))
                        endTime = numel(obj.original.time);
                    end
                    pf_mg_kgfuel = (obj.(source).emissions{emissionIx}.cumulativeSoa_g(endTime)-...
                        obj.(source).emissions{emissionIx}.cumulativeSoa_g(startTime))/...
                        (obj.(source).emissions{emissionIx}.cumulativeCo2_g(endTime)-...
                        obj.(source).emissions{emissionIx}.cumulativeCo2_g(startTime)).*kk.*1e3; % mg/kgfuel
                end
            else
                if(nargin ~= 4)
                    error('Wrong number of arguments.');
                end
                if(isnan(endTime))
                    endTime = numel(obj.original.time);
                end
                % PF = (emittedSoa(end)-emittedSoa(start)) /
                % (emittedCo2(end)-emittedCo2(start))
                pf_mg_kgfuel = (obj.original.emissions.cumulativeSoa_g(endTime)-...
                    obj.original.emissions.cumulativeSoa_g(startTime))/...
                    (obj.original.emissions.cumulativeCo2_g(endTime)-...
                    obj.original.emissions.cumulativeCo2_g(startTime)).*kk.*1e3; % mg/kgfuel
            end
        end
    end
end

