% Example of deconvolution
%   Pauli Simonen, Tampere University, 2023

%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 200 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
lw = 1.5; % Line width
widthCm = 16; % Figure width (cm)
heightCm = 9; % Figure height (cm)

hcBase = 0; % ppm
hcPeak = 25; % ppm
qExhBase = 1e-3; % m3/s
qExhPeak = 1e-2; % m3/s
cycleLength = 200; % s
xlims = [0 100]; % Displayed time span (s)

method = 'standard';

%% Create cycle data and initialize reactors

cycle = struct;
cycle.time = (1:1:cycleLength)';
cycle.hc_ppm = hcBase.*ones(cycleLength,1);
hcPulse = smoothsquare(10,hcBase,hcPeak,1.0);
cycle.hc_ppm(10:10+numel(hcPulse)-1) = hcPulse;

cycle.exhFlow_m3_per_s = qExhBase.*ones(cycleLength,1);
qExhPulse = smoothsquare(10,qExhBase,qExhPeak,1.0);
cycle.exhFlow_m3_per_s(10:10+numel(qExhPulse)-1) = qExhPulse;
cycle.co2_percent = 12.*ones(cycleLength,1);
cycle.speed = NaN;

% Note: the starting SOA concentration must be zero. If it is non-zero (c_0),
% it is assumed that the OFRs have been sampling exhaust with concentration
% of c_0 for a long time, so that the starting concentrations in the OFRs
% are c_0 as well. If such data is used for deconvolution, you need to add
% the long period of sampling c_0 prior to cycle start time for the
% deconvolution to succeed.
cycle.hc_ppm(1) = 0; % HC = 0 -> SOA = 0

cycle = Cycle('A',cycle,soaYield,endBufferLength);

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

cycle.addOfr(dofr);
cycle.addOfr(pam);

rtdLength = 150;

% The maximum number of iteratios should be ~5e8 for deconvolution to
% converge to an optimal solution for the cold- and hot-start NEDC data
% used in the paper. Here, 1e5 does not converge but is rather fast and
% results in quite good solutions.
maxIterations = 1e5;
cycle.deconvolveOfrData('DOFR',rtdLength,maxIterations);
rtdLength = 500;
cycle.deconvolveOfrData('PAM',rtdLength,maxIterations);

%% Plot
figure
plot(cycle.original.tailpipe.soa_g_per_m3)
hold on
plot(cycle.DOFR.tailpipe.soa_g_per_m3_shifted)
plot(cycle.DOFR.tailpipeDeconvoluted.soa_g_per_m3)
legend('Original','Downstream OFR','Deconvolved');
xlim([0 cycleLength]);
title('DOFR');

figure
plot(cycle.original.tailpipe.soa_g_per_m3)
hold on
plot(cycle.PAM.tailpipe.soa_g_per_m3_shifted)
plot(cycle.PAM.tailpipeDeconvoluted.soa_g_per_m3)
legend('Original','Downstream OFR','Deconvolved');
xlim([0 cycleLength]);
title('PAM');
