function [out] = createMonteCarloData(numCycles)
%CREATEMONTECARLODATA Creates synthetic driving cycles.
%   Pauli Simonen, Tampere University, 2023

%% Don't display the warning related to deconvolution:
warning('off','Cycle:deconvolution');

%%
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;
interval = 14; % Duration of short events (s)
longInterval = 400; % Duration of subcycles (s)
methods = {'standard','OFR CO_2','convolution','CVS','averaging'};
percentileMethod = 'OFR CO_2'; % Method to use to calculate data for Fig. S12
prctileIntervals = [1, 10:20:210,250:50:500];
for k=1:numel(prctileIntervals)
    dofrPrctiles{k,1} = [];
    pamPrctiles{k,1} = [];
end

out.percentileIntervals = prctileIntervals;

%% Distributions
hcDistLow = makedist('Normal','mu',1,'sigma',10);
hcDistLow = truncate(hcDistLow,0,10);
hcDistMedium = makedist('Normal','mu',30,'sigma',60);
hcDistMedium = truncate(hcDistMedium,15,200);
hcDistHigh = makedist('Normal','mu',1000,'sigma',1000);
hcDistHigh = truncate(hcDistHigh,500,4000);

co2DistGasoline = makedist('Normal','mu',13,'sigma',2);
co2DistGasoline = truncate(co2DistGasoline,3,14);
co2DistDiesel = makedist('Normal','mu',7,'sigma',6);
co2DistDiesel = truncate(co2DistDiesel,1,14);

%% Initialize flow reactors and chamber
dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

%% Initialize cycles
pfTrue = NaN(numCycles,1);

dofrPfs = struct;
pamPfs = struct;
midValues = zeros(1,numel(methods));
for k=1:numel(methods)
    methodName = strrep(methods{k},' ','');
    dofrPfs.(methodName) = NaN(size(pfTrue));
    pamPfs.(methodName) = NaN(size(pfTrue));
    midValues(k) = k+(k-1)*2;
end

isDiesel = NaN(size(pfTrue));

ratiosDofr = table;
ratiosPam = table;
ratiosDofrLong = table;
ratiosPamLong = table;

cycleDuration = NaN(size(pfTrue));

startTime = now();
disp(['Monte Carlo simulation started at ' datestr(startTime) '.']);

%% Create random cycles
disp('Cycle number: ');
for k=1:numCycles
    cc = createRandomCycle(co2DistGasoline,co2DistDiesel,hcDistLow,hcDistMedium,hcDistHigh);

    isDiesel(k) = cc.isDiesel;

    cycle = Cycle('A', cc, soaYield, endBufferLength);
    cycle.addOfr(dofr);
    cycle.addOfr(pam);

    cycleDuration(k) = cycle.original.cycleLength; 
    

    for s=1:numel(methods)
        % Full cycle PFs:
        methodName = strrep(methods{s},' ','');
        dofrPfs.(methodName)(k,1) = cycle.soaPfBetween(1,numel(cycle.original.time),'DOFR',methods{s});
        pamPfs.(methodName)(k,1) = cycle.soaPfBetween(1,numel(cycle.original.time),'PAM',methods{s});

        % PFs for subcycles (400 s) and shorter events (14 s)
        % Subcycles first:
        if(cycle.original.cycleLength > longInterval)
            binWidth = longInterval;
            bins = struct;
            bins.all.startIx = (1:binWidth:cycle.original.cycleLength-binWidth)';
            bins.all.endIx = bins.all.startIx+binWidth;
            [ofrsBinned, originalBinned] = handleBinnedData(cycle, bins, methods{s});
            % DOFR:
            t = table;
            t.values = log10(ofrsBinned.DOFR.all.pf./originalBinned.all.pf);
            t.x = (midValues(s)-0.5).*ones(size(t.values));
            ratiosDofrLong = [ratiosDofrLong; t];
            % PAM:
            t = table;        
            t.values = log10(ofrsBinned.PAM.all.pf./originalBinned.all.pf);
            t.x = (midValues(s)+0.5).*ones(size(t.values));
            ratiosPamLong = [ratiosPamLong; t];
        end

        % Short events:
        binWidth = interval;
        bins = struct;
        bins.all.startIx = (1:binWidth:cycle.original.cycleLength-binWidth)';
        bins.all.endIx = bins.all.startIx+binWidth;
        [ofrsBinned, originalBinned] = handleBinnedData(cycle, bins, methods{s});
        % DOFR:
        t = table;
        t.values = log10(ofrsBinned.DOFR.all.pf./originalBinned.all.pf);
        t.x = (midValues(s)-0.5).*ones(size(t.values));
        ratiosDofr = [ratiosDofr; t];
        % PAM:
        t = table;        
        t.values = log10(ofrsBinned.PAM.all.pf./originalBinned.all.pf);
        t.x = (midValues(s)+0.5).*ones(size(t.values));
        ratiosPam = [ratiosPam; t];

        if(strcmpi(methods{s},percentileMethod))
            for j=1:numel(prctileIntervals)
                if(prctileIntervals(j) >= cycle.original.cycleLength)
                    break;
                end
                bins = struct;
                bins.all.startIx = (1:prctileIntervals(j):cycle.original.cycleLength-prctileIntervals(j))';
                bins.all.endIx = bins.all.startIx+prctileIntervals(j);
                [ofrsBinned, originalBinned] = handleBinnedData(cycle, bins, methods{s});
                values = log10(ofrsBinned.DOFR.all.pf./originalBinned.all.pf);
                dofrPrctiles{j,1} = [dofrPrctiles{j,1}; values];
                values = log10(ofrsBinned.PAM.all.pf./originalBinned.all.pf);
                pamPrctiles{j,1} = [pamPrctiles{j,1}; values];
            end
        end
    end
    pfTrue(k) = cycle.original.emissions.cumulativePf_mg_per_kgfuel(end);
    
    if(k==1)
        fprintf('%d',k);
    else
        numDigitsPrevious = floor(log10(k-1))+1;
        fprintf([repmat('\b',1,numDigitsPrevious) '%d'],k);
    end
end
fprintf('\r\n');

endTime = now();
disp(['Monte Carlo simulation finished at ' datestr(endTime) '.']);
disp(['Total duration: ' num2str((endTime-startTime)*24*60) ' min.']);

%% Re-enable deconvolution warning:
warning('on','Cycle:deconvolution');

out.isDiesel = isDiesel;
out.pfTrue = pfTrue;
out.dofrPfs = dofrPfs;
out.pamPfs = pamPfs;
out.ratiosDofr = ratiosDofr;
out.ratiosDofrLong = ratiosDofrLong;
out.ratiosPam = ratiosPam;
out.ratiosPamLong = ratiosPamLong;
out.dofrPrctiles = dofrPrctiles;
out.pamPrctiles = pamPrctiles;
out.cycleDuration = cycleDuration;
out.midValues = midValues;

end

