function [cycle] = createRandomCycle(co2DistGasoline,co2DistDiesel,hcDistLow,hcDistMedium,hcDistHigh)
%CREATERANDOMCYCLE Creates a synthetic driving cycle
%   Pauli Simonen, Tampere University, 2023
cycle = struct;

minLength = 240; % 4 min
maxLength = 2400; % 40 min

longProbability = 0.1;

cycleLength = randi([minLength maxLength],1,1);

rr = rand(1);
if(rr < 0.5)
    isDiesel = false;
else
    isDiesel = true;
end
cycle.isDiesel = isDiesel;

%% CO2
co2Min = 2;
co2Max = 14;
co2MinDuration = 2;
co2MaxDuration = 25;
co2MaxDurationLong = 100;
cycle.co2 = zeros(cycleLength,1);
cycle.co2(1) = 0;

%% HC
hcMinDuration = 2;
hcMaxDuration = 25;
hcMaxDurationLong = 100;

hcMax = 3000;
hcMin = 1;

cycle.hc = zeros(cycleLength,1);
cycle.hc(1) = 0;

hcLowProbability = 0.513;
hcMediumProbability = 0.48;
hcHighProbability = 0.007;

if(hcLowProbability+hcMediumProbability+hcHighProbability > 1.001 ||... 
    hcLowProbability+hcMediumProbability+hcHighProbability < 0.999)
    error('HC probabilities do not sum to 1.');
end

%% Qexh
qMin = 0.75e-3;
qMax = 0.08;
qMinDuration = 2;
qMaxDuration = 25;
qMaxDurationLong = 100;
engOffMinDuration = 10;
engOffMaxDuration = 600;
% qZeroProbability = 0.01;
qZeroProbability = 0;
cycle.q = qMin.*ones(cycleLength,1);
cycle.q(1) = 0;

%% First Qexh
startPoint = 2;
cycleFinished = false;
while(~cycleFinished)
    if(rand(1) < qZeroProbability)
        duration = randi([engOffMinDuration engOffMaxDuration],1,1);
        if(duration+startPoint > cycleLength)
            continue; % Cycle doesn't end to engine off condition...
        end
        nextPoint = startPoint + duration;
        cycle.q(startPoint:nextPoint-1) = 0;
        startPoint = nextPoint;
        continue;
    end

    if(rand(1) < longProbability)
        minDuration = qMaxDuration;
        maxDuration = qMaxDurationLong;
    else
        minDuration = qMinDuration;
        maxDuration = qMaxDuration;
    end

    duration = randi([minDuration maxDuration],1,1);
    qLevel = randb(qMin, qMax);
    previousQ = cycle.q(startPoint-1);
    q = smoothstep((1:duration)',previousQ,qLevel,min([3 duration]),1.5);

    if(numel(q)+startPoint > cycleLength)
        q = q(1:cycleLength-startPoint);
        cycleFinished = true;
    end
    duration = numel(q);
    nextPoint = startPoint + duration;

    cycle.q(startPoint:nextPoint-1) = q;

    startPoint = nextPoint;

end
cycle.q(end) = 0;

%% HC
startPoint = 2;
cycleFinished = false;
while(~cycleFinished)
    if(rand(1) < longProbability)
        minDuration = hcMaxDuration;
        maxDuration = hcMaxDurationLong;
    else
        minDuration = hcMinDuration;
        maxDuration = hcMaxDuration;
    end

    duration = randi([minDuration maxDuration],1,1);

    rr = rand(1);
    if(rr < hcLowProbability)
        hcLevel = random(hcDistLow,1,1);
    elseif(rr < hcLowProbability + hcMediumProbability)
        hcLevel = random(hcDistMedium,1,1);
    else
        hcLevel = random(hcDistHigh,1,1);
    end

    % hcLevel = randb(hcMin, hcMax);
    previousHc = cycle.hc(startPoint-1);
    hc = smoothstep((1:duration)',previousHc,hcLevel,min([3 duration]),1.5);

    if(numel(hc)+startPoint > cycleLength)
        hc = hc(1:cycleLength-startPoint);
        cycleFinished = true;
    end
    duration = numel(hc);
    nextPoint = startPoint + duration;

    cycle.hc(startPoint:nextPoint-1) = hc;

    startPoint = nextPoint;

end
cycle.hc(end) = 0;
cycle.hc(cycle.q < 1e-9) = 0;

%% CO2
startPoint = 2;
cycleFinished = false;
while(~cycleFinished)
    if(rand(1) < longProbability)
        minDuration = co2MaxDuration;
        maxDuration = co2MaxDurationLong;
    else
        minDuration = co2MinDuration;
        maxDuration = co2MaxDuration;
    end

    duration = randi([minDuration maxDuration],1,1);

    if(isDiesel)
        co2Level = random(co2DistDiesel,1,1);
    else
        co2Level = random(co2DistGasoline,1,1);
    end

    % co2Level = randb(co2Min, co2Max);
    previousCo2 = cycle.co2(startPoint-1);
    co2 = smoothstep((1:duration)',previousCo2,co2Level,min([3 duration]),1.5);

    if(numel(co2)+startPoint > cycleLength)
        co2 = co2(1:cycleLength-startPoint);
        cycleFinished = true;
    end
    duration = numel(co2);
    nextPoint = startPoint + duration;

    cycle.co2(startPoint:nextPoint-1) = co2;

    startPoint = nextPoint;

end
cycle.co2(end) = 0;
cycle.co2(cycle.q < 1e-9) = 0;

% Correct variable names:
cycle.co2_percent = cycle.co2;
cycle.hc_ppm = cycle.hc;
cycle.exhFlow_m3_per_s = cycle.q;

cycle = rmfield(cycle,{'co2','hc','q'});

% Add required fields:
cycle.speed = NaN;
cycle.time = (1:numel(cycle.co2_percent))';

end


