function [simulatedConc] = simulateRtd(inputConc,rtd,t_shift)
%UNTITLED Simulates signal broadening and delay due to transfer function
%   Returns the concentration 'inputConc' as it would appear
%   downstream of the instrument with transfer function
%   specified in parameter 'rtd'.
% 
%   The transfer function must be defined at 1 second time resolution.
%   
%   The signal convolved with the transfer function is returned as
%   'simulatedConc'.
% 
%   Pauli Simonen, Tampere University, 2023


%% Input checks
[rows,cols] = size(inputConc);
assert(rows == 1 || cols == 1, ['The inputConc vector '...
    'must be a n x 1 or 1 x n vector.']);

[rtdRows,rtdCols] = size(rtd);
assert(rtdRows == 1 || rtdCols == 1, ['The RTD vector '...
    'must be a n x 1 or 1 x n vector.']);

assert(sum(rtd) > 1-10*eps && sum(rtd) < 1+10*eps,...
    'The integral of RTD must equal 1.');

assert(t_shift >= 0, ['Parameter ''t_shift'' must be non-negative. '...
    'Modify the function in order to use negative ''t_shift''.']);


% Change the orientation of vectors if necessary:
if(rows == 1)
    inputConc = inputConc';
end
if(rtdRows == 1)
    rtd = rtd';
end

%% Convolve the signals with the RTD
% Add some pre-simulation data (assuming that conc. has been stable
% with value of the first element for a long time (700 s)):
preSimulationDuration = 700;
inputLength = numel(inputConc);
inputConc = [inputConc(1).*ones(preSimulationDuration,1);...
    inputConc];
convolvedData = conv(rtd,inputConc);
 % Remove the pre-simulation data:
convolvedData = convolvedData(preSimulationDuration:end);

% Assign the convolved data to output vector, so that the length of the
% output vector equals that of the input vector. Apply also the time shift,
% i.e., the simple delay correction:
if(inputLength+round(t_shift) > numel(convolvedData))
    error(['The convolved data is too short to allow the requested '...
        'time shift. Please decrease the time shift or increase the '...
        'length of the RTD vector.']);
end
simulatedConc = convolvedData(round(t_shift+1):...
    (inputLength+round(t_shift)));

end

