%   Pauli Simonen, Tampere University, 2023
%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
widthCm = 16; % Figure width (cm)
heightCm = 9; % Figure height (cm)

method = 'standard'; % The method to use for calculating OFR emissions

nBins = 11; % Number of bins in histograms

%% Load data and initialize reactors
load ./data/hotNedc.mat hotNedc
load ./data/coldNedc.mat coldNedc

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

hot = Cycle('Hot NEDC', hotNedc, soaYield, endBufferLength);
cold = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);

% Add reactors to the cycle:
hot.addOfr(dofr);
hot.addOfr(pam);
cold.addOfr(dofr);
cold.addOfr(pam);

clear dofr pam soaYield endBufferLength hotNedc coldNedc;

%% Divide NEDC cycle into bins
% Load theoretical speed and adjust time to match recorded speed:
load ./data/nedcSpeed.mat nedcSpeed
nedcSpeed.time = nedcSpeed.time + 1;

bins = struct;
[bins.accelerations, bins.idles, bins.constSpeeds, bins.decelerations] =...
    binCycleData(nedcSpeed.time, nedcSpeed.speed);

[ofrsBinnedCold, originalBinnedCold] = handleBinnedData(cold, bins, method);
[ofrsBinnedHot, originalBinnedHot] = handleBinnedData(hot, bins, method);

%% Plot figure
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Color = 'w';

t = tiledlayout(2,4);
t.Padding = 'none';
t.TileSpacing = 'compact';

reactors = fields(ofrsBinnedCold);
cases = {'accelerations','constSpeeds','idles','decelerations'};
caseTitles = {'Acceleration','Constant speed','Idle','Deceleration'};
cycles = {'coldNedc','hotNedc'};
cycleTitles = {'CS','HS'};
figLabels = {'(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'};
ofrs = struct;
ofrs.coldNedc = ofrsBinnedCold;
ofrs.hotNedc = ofrsBinnedHot;
drivingCycles.coldNedc = originalBinnedCold;
drivingCycles.hotNedc = originalBinnedHot;

% Find minimum and maximum ratios to decide suitable bin edges:
maxValue = 0;
minValue = 0;
for k=1:numel(reactors)
    for j=1:numel(cases)
        currentMax = max([log10(ofrs.coldNedc.(reactors{k}).(cases{j}).pf./...
            drivingCycles.coldNedc.(cases{j}).pf);...
            log10(ofrs.hotNedc.(reactors{k}).(cases{j}).pf./...
            drivingCycles.hotNedc.(cases{j}).pf)]);
        currentMin = min([log10(ofrs.coldNedc.(reactors{k}).(cases{j}).pf./...
            drivingCycles.coldNedc.(cases{j}).pf);...
            log10(ofrs.hotNedc.(reactors{k}).(cases{j}).pf./...
            drivingCycles.hotNedc.(cases{j}).pf)]);
        if(currentMax > maxValue)
            maxValue = currentMax;
        end
        if(currentMin < minValue)
            minValue = currentMin;
        end
    end
end
maxAbs = max(abs([minValue maxValue]));
minValue = -maxAbs;
maxValue = maxAbs;
edges = logspace(minValue,maxValue,nBins+1);
xlims = [0.08 10];

% Plot all panels:
ii = 1;
for c=1:numel(cycles)
    for j=1:numel(cases)
        nexttile;
        for k=1:numel(reactors)
            histogram(ofrs.(cycles{c}).(reactors{k}).(cases{j}).pf./drivingCycles.(cycles{c}).(cases{j}).pf,edges,'Normalization','probability');
            hold on;
        end
        set(gca,'xscale','log');
        ylim([0 0.8]);
        xlim(xlims);
        set(gca,'XTick',[1e-1 1e0 1e1]);
        grid on;
        title([figLabels{ii} ' ' cycleTitles{c} ', ' caseTitles{j}]);
        set(gca,'TitleFontWeight','normal');
        set(gca,'TitleFontSizeMultiplier',1.0);
        ax(ii) = gca;
        if(ii > 4)
            ax(ii).XLabel.Interpreter = 'latex';
            % ax(ii).XLabel.String = '$\fontfamily{Arial}\selectfont\frac{\mathrm{Reactor\ PF}}{\mathrm{Reference\ PF}}$';
            ax(ii).XLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
        end
        if(ii == 1 || ii == 5)        
            ax(ii).YLabel.String = 'Fraction';
        end
        set(gca,'FontSize',fs);
        set(gca,'LooseInset',[0 0 0 0]);
        ii = ii + 1;
    end
end

axes(ax(2));
legh = legend(reactors{1}, reactors{2},'Location','North');

%% Export pdf:
if(isExport)
    exportgraphics(gcf,'./exportedFigures/fig4.pdf');
end

clear isExport

%% Clear workspace
clear ax bins binWidth c cases caseTitles cold currentMax currentMin;
clear cycles cycleTitles drivingCycles edges fh figLabels fs;
clear heightCm hot ii j k legh lw maxAbs maxValue method minValue;
clear nBins nedcSpeed ofrs ofrsBinnedCold ofrsBinnedHot originalBinnedCold;
clear originalBinnedHot reactors t widthCm xlims;