%   Pauli Simonen, Tampere University, 2023
%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size

widthCm = 16; % Figure width (cm)
heightCm = 12.5; % Figure height (cm)

subCycles = {'CSUDC','HUDC','EUDC','Full cycle'};
subCycleTimes = [1 392;...
                392 788;...
                788 NaN;... % NaN -> PF is calculated between 788 and end
                1 NaN];


%% Deconvolve data or load deconvolved data
% Cold-start cycle:
file2load = './data/deconvolutions/dc_2023_10_31_20_00_coldNedc.mat';
if(~isfile(file2load))
    disp('Pre-calculated deconvolved data not found - deconvolving now...');
    maxIterations = 5e8;

    load ./data/coldNedc.mat coldNedc
    
    dofr = Ofr('DOFR');
    dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.
    
    pam = Ofr('PAM');
    pam.setRtd(calcPamRtd(700));
    
    cold = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);
    
    % Add reactors to the cycle:
    cold.addOfr(dofr);
    cold.addOfr(pam);
    
    clear dofr pam coldNedc;
    
    cold.deconvolveOfrData('DOFR',150,maxIterations);
    cold.deconvolveOfrData('PAM',500,maxIterations);

    if(isSave)
        fnam = ['dc_' datestr(now(), 'yyyy_mm_dd_HH_MM')];
        save(['./data/deconvolutions/' fnam '_coldNedc.mat'],'cold');
        disp(['File ./data/deconvolutions/' fnam '_coldNedc.mat saved.']);
    end
    clear maxIterations
else
    disp('Loading the deconvolved data...');
    load(file2load);
    disp('Deconvolved data loaded.');

    clear file2load
end

% Hot-start cycle:
file2load = './data/deconvolutions/dc_2023_10_31_21_04_hotNedc.mat';
if(~isfile(file2load))
    disp('Pre-calculated deconvolved data not found - deconvolving now...');
    maxIterations = 5e8;

    load ./data/hotNedc.mat hotNedc
    
    dofr = Ofr('DOFR');
    dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.
    
    pam = Ofr('PAM');
    pam.setRtd(calcPamRtd(700));
    
    hot = Cycle('Hot NEDC', hotNedc, soaYield, endBufferLength);
    
    % Add reactors to the cycle:
    hot.addOfr(dofr);
    hot.addOfr(pam);
    
    clear dofr pam soaYield endBufferLength hotNedc;
    
    hot.deconvolveOfrData('DOFR',150,maxIterations);
    hot.deconvolveOfrData('PAM',500,maxIterations);

    if(isSave)
        fnam = ['dc_' datestr(now(), 'yyyy_mm_dd_HH_MM')];
        save(['./data/deconvolutions/' fnam '_hotNedc.mat'],'hot');
        disp(['File ./data/deconvolutions/' fnam '_hotNedc.mat saved.']);
    end
    clear maxIterations
else
    disp('Loading the deconvolved data...');
    load(file2load);
    disp('Deconvolved data loaded.');
    
    clear file2load
end

clear endBufferLength soaYield isSave

%% Calculate the OFR-to-True ratios:
barDataDofrHot = NaN(numel(subCycles),numel(hot.DOFR.emissions));
barDataPamHot = NaN(numel(subCycles),numel(hot.DOFR.emissions));
barDataDofrCold = NaN(numel(subCycles),numel(hot.DOFR.emissions));
barDataPamCold = NaN(numel(subCycles),numel(hot.DOFR.emissions));

dofrLabels = {};
pamLabels = {};
legStr = {};

for k=1:numel(hot.DOFR.emissions)
    method = hot.DOFR.emissions{k}.method;
    % Skip CVS no delay correction method:
    if(strcmpi(method,'CVS_noDelayCorrection'))
        continue;
    end
    if(strcmpi(method,'OFR CO_2'))
        ix = k; % For displaying the percentiles later
    end
    for j=1:numel(subCycles)
        barDataDofrHot(j,k) =...
            hot.soaPfBetween(subCycleTimes(j,1),subCycleTimes(j,2),'DOFR',method)./...
            hot.soaPfBetween(subCycleTimes(j,1),subCycleTimes(j,2),'original');
        barDataDofrCold(j,k) =...
            cold.soaPfBetween(subCycleTimes(j,1),subCycleTimes(j,2),'DOFR',method)./...
            cold.soaPfBetween(subCycleTimes(j,1),subCycleTimes(j,2),'original');
        barDataPamHot(j,k) =...
            hot.soaPfBetween(subCycleTimes(j,1),subCycleTimes(j,2),'PAM',method)./...
            hot.soaPfBetween(subCycleTimes(j,1),subCycleTimes(j,2),'original');
        barDataPamCold(j,k) =...
            cold.soaPfBetween(subCycleTimes(j,1),subCycleTimes(j,2),'PAM',method)./...
            cold.soaPfBetween(subCycleTimes(j,1),subCycleTimes(j,2),'original');
        if(k==1)
            dofrLabels{end+1} = ['DOFR\newline' subCycles{j}];
            pamLabels{end+1} = ['PAM\newline' subCycles{j}];
        end
    end
    method(1) = upper(method(1)); % Capitalize first letter
    legStr{end+1} = method;
end

clear k j method subCycleTimes subCycles
%% Plot
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Color = 'w';

t = tiledlayout(2,1);
t.Padding = 'none';
t.TileSpacing = 'compact';

% Panel (a):
nexttile;
bh1 = bar([barDataDofrHot; barDataPamHot]);
set(gca,'XTickLabel',[dofrLabels, pamLabels]);
legh = legend(legStr,'Location','SouthWest','NumColumns',2);
legh.BoxFace.ColorType = 'truecoloralpha';
legh.BoxFace.ColorData=uint8(255*[1 1 1 0.9]');
ylabel('Reactor PF / Reference PF');
set(gca,'YGrid','on');
title('(a) Hot-start NEDC');
set(gca,'FontSize',fs);
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

% Panel (b):
nexttile;
bh2 = bar([barDataDofrCold; barDataPamCold]);
set(gca,'XTickLabel',[dofrLabels, pamLabels]);
ylabel('Reactor PF / Reference PF');
set(gca,'YGrid','on');
title('(b) Cold-start NEDC');
set(gca,'FontSize',fs);
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

clear fh t bh1 legh bh2;
clear dofrLabels pamLabels legStr;
clear widthCm heightCm;

%% Export pdf:
if(isExport)
    exportgraphics(gcf,'./exportedFigures/fig5.pdf');
end

clear isExport

%% Display percentiles for Table 1
dofrRatios = [barDataDofrCold(1:end-1,ix); barDataDofrHot(1:end-1,ix)];
pamRatios = [barDataPamCold(1:end-1,ix); barDataPamHot(1:end-1,ix)];

d = struct;
d.median = median(dofrRatios);
d.p01 = prctile(dofrRatios,1);
d.p25 = prctile(dofrRatios,25);
d.p75 = prctile(dofrRatios,75);
d.p99 = prctile(dofrRatios,99);

p = struct;
p.median = median(pamRatios);
p.p01 = prctile(pamRatios,1);
p.p25 = prctile(pamRatios,25);
p.p75 = prctile(pamRatios,75);
p.p99 = prctile(pamRatios,99);

disp('Subcycles, OFR_CO2 method:');
disp(['DOFR: Median: ' num2str(round(d.median,2)),...
    ', P01: ' num2str(round(d.p01,2)),...
    ', P99: ' num2str(round(d.p99,2)),...
    ', P25: ' num2str(round(d.p25,2)),...
    ', P75: ' num2str(round(d.p75,2))]);
disp(['PAM: Median: ' num2str(round(p.median,2)),...
    ', P01: ' num2str(round(p.p01,2)),...
    ', P99: ' num2str(round(p.p99,2)),...
    ', P25: ' num2str(round(p.p25,2)),...
    ', P75: ' num2str(round(p.p75,2))]);

%% Clear workspace
clear t fh fs lw hot cold d dofrRatios ix p pamRatios;
clear barDataDofrCold barDataDofrHot barDataPamCold barDataPamHot;