%   Pauli Simonen, Tampere University, 2023
%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
ww = 0.8; % Whisker width
bw = 0.6; % Box width
widthCm = 8.3; % Figure width (cm)
heightCm = 12; % Figure height (cm)

%% Deconvolve data or load deconvolved data
% Cold-start cycle:
file2load = './data/deconvolutions/dc_2023_10_31_20_00_coldNedc.mat';
if(~isfile(file2load))
    disp('Pre-calculated deconvolved data not found - deconvolving now...');
    maxIterations = 5e8;

    load ./data/coldNedc.mat coldNedc
    
    dofr = Ofr('DOFR');
    dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.
    
    pam = Ofr('PAM');
    pam.setRtd(calcPamRtd(700));
    
    cold = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);
    
    % Add reactors to the cycle:
    cold.addOfr(dofr);
    cold.addOfr(pam);
    
    clear dofr pam coldNedc;
    
    cold.deconvolveOfrData('DOFR',150,maxIterations);
    cold.deconvolveOfrData('PAM',500,maxIterations);

    if(isSave)
        fnam = ['dc_' datestr(now(), 'yyyy_mm_dd_HH_MM')];
        save(['./data/deconvolutions/' fnam '_coldNedc.mat'],'cold');
        disp(['File ./data/deconvolutions/' fnam '_coldNedc.mat saved.']);
    end
    clear maxIterations
else
    disp('Loading the deconvolved data...');
    load(file2load);
    disp('Deconvolved data loaded.');

    clear file2load
end

% Hot-start cycle:
file2load = './data/deconvolutions/dc_2023_10_31_21_04_hotNedc.mat';
if(~isfile(file2load))
    disp('Pre-calculated deconvolved data not found - deconvolving now...');
    maxIterations = 5e8;

    load ./data/hotNedc.mat hotNedc
    
    dofr = Ofr('DOFR');
    dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.
    
    pam = Ofr('PAM');
    pam.setRtd(calcPamRtd(700));
    
    hot = Cycle('Hot NEDC', hotNedc, soaYield, endBufferLength);
    
    % Add reactors to the cycle:
    hot.addOfr(dofr);
    hot.addOfr(pam);
    
    clear dofr pam soaYield endBufferLength hotNedc;
    
    hot.deconvolveOfrData('DOFR',150,maxIterations);
    hot.deconvolveOfrData('PAM',500,maxIterations);

    if(isSave)
        fnam = ['dc_' datestr(now(), 'yyyy_mm_dd_HH_MM')];
        save(['./data/deconvolutions/' fnam '_hotNedc.mat'],'hot');
        disp(['File ./data/deconvolutions/' fnam '_hotNedc.mat saved.']);
    end
    clear maxIterations
else
    disp('Loading the deconvolved data...');
    load(file2load);
    disp('Deconvolved data loaded.');
    
    clear file2load
end

clear endBufferLength soaYield isSave

%% Divide cycles into 14 s bins and calculate ratios for each bin:
bins = struct;
bins.all.startIx = (1:14:hot.original.cycleLength-14)';
bins.all.endIx = bins.all.startIx+13;

ratiosDofrHot = table;
ratiosDofrCold = table;
ratiosPamHot = table;
ratiosPamCold = table;
midValues = [];
methodLabels = {};

for k=1:numel(hot.DOFR.emissions)
    method = hot.DOFR.emissions{k}.method;
    % Skip CVS without delay correction:
    if(strcmpi(method,'CVS_noDelayCorrection'))
        continue;
    end
    [ofrsBinnedHot, originalBinnedHot] = handleBinnedData(hot, bins, method);
    [ofrsBinnedCold, originalBinnedCold] = handleBinnedData(cold, bins, method);

    midValues(k) = k+(k-1)*2;

    t = table;        
    t.values = ofrsBinnedHot.DOFR.all.pf./originalBinnedHot.all.pf;
    dofrHot = t.values;
    t.x = (midValues(k)-0.5).*ones(size(t.values));
    ratiosDofrHot = [ratiosDofrHot; t];

    t = table;        
    t.values = ofrsBinnedCold.DOFR.all.pf./originalBinnedCold.all.pf;
    dofrCold = t.values;
    t.x = (midValues(k)-0.5).*ones(size(t.values));
    ratiosDofrCold = [ratiosDofrCold; t];

    t = table;        
    t.values = ofrsBinnedHot.PAM.all.pf./originalBinnedHot.all.pf;
    pamHot = t.values;
    t.x = (midValues(k)+0.5).*ones(size(t.values));
    ratiosPamHot = [ratiosPamHot; t];

    t = table;        
    t.values = ofrsBinnedCold.PAM.all.pf./originalBinnedCold.all.pf;
    pamCold = t.values;
    t.x = (midValues(k)+0.5).*ones(size(t.values));
    ratiosPamCold = [ratiosPamCold; t];

    method(1) = upper(method(1));
    methodLabels{end+1} = method;

    % Combine cold- and hot-start ratios for OFR CO_2 method:
    if(strcmp(method,'OFR CO_2'))
        ratiosDofr = [dofrCold; dofrHot];
        ratiosPam = [pamCold; pamHot];
    end
end
    
%% Plot figure
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Color = 'w';

t = tiledlayout(2,1);
t.Padding = 'none';
t.TileSpacing = 'compact';

% Panel (a):
nexttile;
b1 = boxchart_ps(ratiosDofrHot,'blue',ww);
b1.BoxWidth = bw;
hold on
b2 = boxchart_ps(ratiosPamHot,'red',ww);
b2.BoxWidth = bw;
set(gca,'XTick',midValues);
set(gca,'XTickLabel',methodLabels);
set(gca,'YScale','log');
ylim([1e-1 1e1]);
ax = gca;
ax.YLabel.Interpreter = 'latex';               
ax.YLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
grid on;
title('(a) Hot-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
legh1 = legend([b1, b2],'DOFR','PAM','Location','SouthEast');
legh1.ItemTokenSize(1) = 15;
set(gca,'FontSize',fs);
set(gca,'Box','on');
xlim([midValues(1)-2 midValues(end)+2]);

% Panel (b):
nexttile;
b1 = boxchart_ps(ratiosDofrCold,'blue',ww);
b1.BoxWidth = bw;
hold on
b2 = boxchart_ps(ratiosPamCold,'red',ww);
b2.BoxWidth = bw;
set(gca,'XTick',midValues);
set(gca,'XTickLabel',methodLabels);
set(gca,'YScale','log');
ylim([1e-1 1e1]);
ax = gca;
ax.YLabel.Interpreter = 'latex';               
ax.YLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
grid on;
title('(b) Cold-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
legh2 = legend([b1, b2],'DOFR','PAM','Location','SouthEast');
legh2.ItemTokenSize(1) = 15;
set(gca,'FontSize',fs);
set(gca,'Box','on');
xlim([midValues(1)-2 midValues(end)+2]);

legh1.Position(2) = legh1.Position(2)-0.015;
legh2.Position(2) = legh2.Position(2)-0.015;

%% Export pdf:
if(isExport)
    exportgraphics(gcf,'./exportedFigures/fig6.pdf');
end

clear isExport

%% Print medians and percentiles for Table 1:
d = struct;
d.median = median(ratiosDofr);
d.p01 = prctile(ratiosDofr,1);
d.p25 = prctile(ratiosDofr,25);
d.p75 = prctile(ratiosDofr,75);
d.p99 = prctile(ratiosDofr,99);

p = struct;
p.median = median(ratiosPam);
p.p01 = prctile(ratiosPam,1);
p.p25 = prctile(ratiosPam,25);
p.p75 = prctile(ratiosPam,75);
p.p99 = prctile(ratiosPam,99);

disp('14 s events, OFR_CO2 method:');
disp(['DOFR: Median: ' num2str(round(10^d.median,2)),...
    ', P01: ' num2str(round(10^d.p01,2)),...
    ', P99: ' num2str(round(10^d.p99,2)),...
    ', P25: ' num2str(round(10^d.p25,2)),...
    ', P75: ' num2str(round(10^d.p75,2))]);
disp(['PAM: Median: ' num2str(round(10^p.median,2)),...
    ', P01: ' num2str(round(10^p.p01,2)),...
    ', P99: ' num2str(round(10^p.p99,2)),...
    ', P25: ' num2str(round(10^p.p25,2)),...
    ', P75: ' num2str(round(10^p.p75,2))]);

%% Clear workspace
clear ax b1 b2 bins bw cold d dofrCold dofrHot fh;
clear fs heightCm hot k legh1 legh2 lw method methodLabels midValues;
clear ofrsBinnedCold ofrsBinnedHot originalBinnedCold originalBinnedHot;
clear p pamCold pamHot ratiosDofr ratiosDofrCold ratiosDofrHot ratiosPam;
clear ratiosPamCold ratiosPamHot t widthCm ww;

