%   Pauli Simonen, Tampere University, 2023
%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
lw = 1.5; % Line width
widthCm = 16; % Figure width (cm)
heightCm = 9; % Figure height (cm)

hcBase = 2; % ppm
hcPeak = 25; % ppm
qExhBase = 1e-3; % m3/s
qExhPeak = 1e-2; % m3/s
cycleLength = 200; % s
xlims = [0 100]; % Displayed time span (s)

method = 'standard';

%% Create cycle data and initialize reactors

delay = 0;
cycle1 = struct;
cycle1.time = (1:1:cycleLength)';
cycle1.hc_ppm = hcBase.*ones(cycleLength,1);
hcPulse = smoothsquare(10,hcBase,hcPeak,1.0);
cycle1.hc_ppm(10:10+numel(hcPulse)-1) = hcPulse;

cycle1.exhFlow_m3_per_s = qExhBase.*ones(cycleLength,1);
qExhPulse = smoothsquare(10,qExhBase,qExhPeak,1.0);
cycle1.exhFlow_m3_per_s(10+delay:10+delay+numel(qExhPulse)-1) = qExhPulse;
cycle1.co2_percent = 12.*ones(cycleLength,1);
cycle1.speed = NaN;

delay = 18;
cycle2 = struct;
cycle2.time = (1:1:cycleLength)';
cycle2.hc_ppm = hcBase.*ones(cycleLength,1);
hcPulse = smoothsquare(10,hcBase,hcPeak,1.0);
cycle2.hc_ppm(10:10+numel(hcPulse)-1) = hcPulse;
cycle2.exhFlow_m3_per_s = qExhBase.*ones(cycleLength,1);
qExhPulse = smoothsquare(10,qExhBase,qExhPeak,1.0);
cycle2.exhFlow_m3_per_s(10+delay:10+delay+numel(qExhPulse)-1) = qExhPulse;
cycle2.co2_percent = 12.*ones(cycleLength,1);
cycle2.speed = NaN;

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

cycle1 = Cycle('A', cycle1, soaYield, endBufferLength);
cycle2 = Cycle('B', cycle2, soaYield, endBufferLength);

% Add reactors to the cycle:
cycle1.addOfr(dofr);
cycle1.addOfr(pam);
cycle2.addOfr(dofr);
cycle2.addOfr(pam);


clear dofr pam soaYield endBufferLength;
% clear dofrDeconv pamDeconv;

%% Resolve OFR emission index
emissionIx = NaN;

for k=1:numel(cycle1.(cycle1.ofrNames{1}).emissions)
    if(strcmp(cycle1.(cycle1.ofrNames{1}).emissions{k}.method, method))
        emissionIx = k;
        break;
    end
end
if(isnan(emissionIx))
    error(['Calculation method ''' method ''' not found.']);
end

clear k method

%% Create figure
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.PaperPositionMode = 'auto';
fh.Color = 'w';

blueC = [0 114 189]./255;
redC = [217 83 25]./255;
grayC = [0.3 0.3 0.3];

t = tiledlayout(2,2);
t.TileSpacing = 'compact';
t.Padding = 'none';

%% Panel (a)
nexttile;
hv = plot(cycle1.original.time,cycle1.original.tailpipe.soa_g_per_m3.*1e3,'LineWidth',lw,'Color',grayC);
hold on;
hd = plot(cycle1.DOFR.time,cycle1.DOFR.tailpipe.soa_g_per_m3_shifted.*1e3,'LineWidth',lw,'Color',blueC,'LineStyle','-');
hp = plot(cycle1.PAM.time,cycle1.PAM.tailpipe.soa_g_per_m3_shifted.*1e3,'LineWidth',lw,'Color',redC,'LineStyle',':');
ylabel('SOA (mg m^{-3})');
xlim(xlims);
ylim([0 3]);

yyaxis right
hq = plot(cycle1.original.time,cycle1.original.exhFlow_m3_per_s.*1e3,'LineWidth',lw,'Color',redC);
ylabel('Q_{exh} (L s^{-1})');
xlim(xlims);
set(gca,'FontSize',fs);
set(gca,'YColor','k');

yyaxis left;
grid on;
legh1 = legend([hq,hv,hd,hp],'Q_{exh}', '[SOA]_{Ref}', '[SOA]_{DOFR}', '[SOA]_{PAM}');
legh1.ItemTokenSize(1) = 15;
legh1.BoxFace.ColorType = 'truecoloralpha';
legh1.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
title('(a)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

%% Panel (b)
nexttile(3);
plot(cycle1.original.time,cycle1.original.emissions.cumulativeSoa_g.*1e3,'LineWidth',lw,'Color',grayC);
hold on;
plot(cycle1.DOFR.time,cycle1.DOFR.emissions{emissionIx}.cumulativeSoa_g.*1e3,'LineWidth',lw,'Color',blueC,'LineStyle','-');
plot(cycle1.PAM.time,cycle1.PAM.emissions{emissionIx}.cumulativeSoa_g.*1e3,'LineWidth',lw,'Color',redC,'LineStyle',':');
xlim(xlims);
ylim([0 0.225]);
legh2 = legend('Reference','DOFR','PAM','Location','NorthEast');
legh2.ItemTokenSize(1) = 15;
legh2.BoxFace.ColorType = 'truecoloralpha';
legh2.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
ylabel('Emitted SOA (mg)');
set(gca,'FontSize',fs);
title('(b)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
grid on;
xlabel('Time (s)');

%% Panel (c)
nexttile(2);
hv = plot(cycle2.original.time,cycle2.original.tailpipe.soa_g_per_m3.*1e3,'LineWidth',lw,'Color',grayC);
hold on;
hd = plot(cycle2.DOFR.time,cycle2.DOFR.tailpipe.soa_g_per_m3_shifted.*1e3,'LineWidth',lw,'Color',blueC,'LineStyle','-');
hp = plot(cycle2.PAM.time,cycle2.PAM.tailpipe.soa_g_per_m3_shifted.*1e3,'LineWidth',lw,'Color',redC,'LineStyle',':');
ylabel('SOA (mg m^{-3})');
xlim(xlims);
ylim([0 30]);

yyaxis right
hq = plot(cycle2.original.time,cycle2.original.exhFlow_m3_per_s.*1e3,'LineWidth',lw,'Color',redC);
ylabel('Q_{exh} (L s^{-1})');
xlim(xlims);
set(gca,'FontSize',fs);
set(gca,'YColor','k');

yyaxis left;
grid on;
ylim([0 3]);
legh1 = legend([hq,hv,hd,hp],'Q_{exh}', '[SOA]_{Ref}', '[SOA]_{DOFR}', '[SOA]_{PAM}');
legh1.ItemTokenSize(1) = 15;
legh1.BoxFace.ColorType = 'truecoloralpha';
legh1.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
title('(c)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

%% Panel (d)
nexttile(4);
plot(cycle2.original.time,cycle2.original.emissions.cumulativeSoa_g.*1e3,'LineWidth',lw,'Color',grayC);
hold on;
plot(cycle2.DOFR.time,cycle2.DOFR.emissions{emissionIx}.cumulativeSoa_g.*1e3,'LineWidth',lw,'Color',blueC,'LineStyle','-');
plot(cycle2.PAM.time,cycle2.PAM.emissions{emissionIx}.cumulativeSoa_g.*1e3,'LineWidth',lw,'Color',redC,'LineStyle',':');
ylims = ylim;
ylim([0 ylims(2)]);
ylims = ylim;
xlim(xlims);
ylim(ylims);
legh2 = legend('Reference','DOFR','PAM','Location','NorthWest');
legh2.ItemTokenSize(1) = 15;
legh2.BoxFace.ColorType = 'truecoloralpha';
legh2.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
ylabel('Emitted SOA (mg)');
set(gca,'FontSize',fs);
title('(d)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
grid on;
xlabel('Time (s)');

%% Export
if(isExport)
    exportgraphics(gcf,'./exportedFigures/fig7.pdf');
end
clear isExport

%% Clear the workspace
clear blueC redC grayC cycle1 cycle2 cycleLength delay emissionIx;
clear fh fs hcBase hcPeak hcPulse hd heightCm hp hq hv legh1 legh2 lw;
clear qExhBase qExhPeak qExhPulse t widthCm xlims ylims;