%   Pauli Simonen, Tampere University, 2023
%%
isExport = true;
isSave = true; % Save the simulated cycles?


%% Create/load synthetic cycles
file2load = './data/monteCarlo/mc_2023_10_26_20_18_n10000.mat';

if(~isfile(file2load))
    disp('Pre-calculated synthetic data not found. Creating new synthetic data...');
    numCycles = 10000;
    o = createMonteCarloData(numCycles);
    isConverted = false;
    disp('Synthetic data created.');
    if(isSave)
        fnam = ['mc_' datestr(now(), 'yyyy_mm_dd_HH_MM') '_n' num2str(numCycles)];
        save(['./data/monteCarlo/' fnam '.mat'],'o');
        disp(['File ./data/monteCarlo/' fnam '.mat saved.']);
    end
else
    disp('Loading the synthetic data...');
    o = load(file2load);
    o = o.o;
    isConverted = false;
    disp('Synthetic data loaded.');
end

%%
methods = fields(o.dofrPfs);
selectedMethods = {'standard','OFRCO_2','convolution'};
methodLabels1 = {};
methodLabels2 = {};

for k=1:numel(selectedMethods)
    methodLabels1{k} = selectedMethods{k};
    if(strcmpi(methodLabels1{k},'OFRCO_2'))
        methodLabels1{k} = 'OFR CO_2';
    end
    methodLabels1{k}(1) = upper(methodLabels1{k}(1));
end

for k=1:numel(methods)
    methodLabels2{k} = methods{k};
    if(strcmpi(methodLabels2{k},'OFRCO_2'))
        methodLabels2{k} = 'OFR CO_2';
    end
    methodLabels2{k}(1) = upper(methodLabels2{k}(1));
end

minRatio = 0;
maxRatio = 0;

for k=1:numel(selectedMethods)
    minVal = min([log10(o.dofrPfs.(selectedMethods{k})./o.pfTrue);...
        log10(o.pamPfs.(selectedMethods{k})./o.pfTrue)]);
    maxVal = max([log10(o.dofrPfs.(selectedMethods{k})./o.pfTrue);...
        log10(o.pamPfs.(selectedMethods{k})./o.pfTrue)]);
    if(minVal < minRatio)
        minRatio = minVal;
    end
    if(maxVal > maxRatio)
        maxRatio = maxVal;
    end
end
    
numBins = 150;
% binWidth = (maxVal-minVal)/numBins;
% edges = zeros(numBins+1,1);
% midpoints = zeros(numBins,1);
% edges(1) = minVal;
% for k=1:numBins
%     edges(k+1) = edges(k)+binWidth;
%     midpoints(k) = (edges(k+1)+edges(k))/2;
% end

edges = logspace(minVal,maxVal,numBins+1);
midpoints = geomean([edges(1:end-1); edges(2:end)]);

fs = 8;
widthCm = 16;
heightCm = 9;

%% Convert logarithmic ratios to linear before plotting
% By default, the ratios in the simulation output file (e.g.
% o.ratiosDofr.values) is log10(DOFR PF / Reference PF). For displaying the
% data, let's convert this to (DOFR PF / Reference PF):
if(~isConverted)
    o.ratiosDofr.values = 10.^(o.ratiosDofr.values);
    o.ratiosDofrLong.values = 10.^(o.ratiosDofrLong.values);
    o.ratiosPam.values = 10.^(o.ratiosPam.values);
    o.ratiosPamLong.values = 10.^(o.ratiosPamLong.values);
    isConverted = true;
end

%%
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Color = 'w';
t = tiledlayout(2,2);
t.Padding = 'none';
t.TileSpacing = 'compact';

nexttile;
for k=1:numel(selectedMethods)
    nDofr = histcounts(o.dofrPfs.(selectedMethods{k})./o.pfTrue,'BinEdges',edges,'Normalization','probability');
    plot(midpoints,nDofr,'LineWidth',1.2);
    hold on;
end
legh = legend(methodLabels1);
legh.ItemTokenSize(1) = 15;

set(gca,'FontSize',fs);
title('(a) Full cycles, DOFR');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
set(gca,'XScale','log')
grid on;
xlim([10^(-0.4) 10^0.4])
ylabel('Probability');
ax = gca;
ax.XLabel.Interpreter = 'latex';               
ax.XLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';

nexttile;
for k=1:numel(selectedMethods)
    nPam = histcounts(o.pamPfs.(selectedMethods{k})./o.pfTrue,'BinEdges',edges,'Normalization','probability');
    plot(midpoints,nPam,'LineWidth',1.2);
    hold on;
end
legh = legend(methodLabels1);
legh.ItemTokenSize(1) = 15;

set(gca,'FontSize',fs);
title('(b) Full cycles, PAM');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
set(gca,'XScale','log')
grid on;
xlim([10^(-0.4) 10^0.4])
ylabel('Probability');
ax = gca;
ax.XLabel.Interpreter = 'latex';               
ax.XLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';

nexttile;
d = struct;
p = struct;
[b1, d.long] = boxchart_ps(o.ratiosDofrLong,'blue',0.75);
b1.BoxWidth = 0.55;
hold on
[b2, p.long] = boxchart_ps(o.ratiosPamLong,'red',0.75);
b2.BoxWidth = 0.55;
set(gca,'XTick',o.midValues);
set(gca,'XTickLabel',methodLabels2);
xlim([o.midValues(1)-1 o.midValues(end)+1])
set(gca,'YScale','log');
grid on
maxLim = 10^(ceil(max(log10([o.ratiosDofrLong.values; o.ratiosPamLong.values]))/0.5)*0.5);
minLim = 10^(floor(min(log10([o.ratiosDofrLong.values; o.ratiosPamLong.values])/0.5))*0.5);
ylim([minLim maxLim]);
set(gca,'FontSize',fs);
ax = gca;
ax.YLabel.Interpreter = 'latex';               
ax.YLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
set(gca,'Box','on');
legh3 = legend([b1,b2],'DOFR','PAM','Location','NorthWest','NumColumns',2);
legh3.ItemTokenSize(1) = 15;
legh3.ItemTokenSize(2) = 10;
legh3.BoxFace.ColorType = 'truecoloralpha';
legh3.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
title('(c) Subcycles (400 s)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
% De-rotate and re-rotate x-axis labels so Matlab doesn't place the labels
% outside the plot area:
rotation = get(gca,'XTickLabelRotation');
set(gca,'XTickLabelRotation',0);
set(gca,'XTickLabelRotation',rotation);
legh3.Position(4) = 0.045;
legh3.Position(1) = legh3.Position(1) - 0.015;
legh3.Position(2) = legh3.Position(2) + 0.036;


nexttile;
[b1, d.short] = boxchart_ps(o.ratiosDofr,'blue',0.75);
b1.BoxWidth = 0.55;
hold on
[b2, p.short] = boxchart_ps(o.ratiosPam,'red',0.75);
b2.BoxWidth = 0.55;
set(gca,'XTick',o.midValues);
set(gca,'XTickLabel',methodLabels2);
xlim([o.midValues(1)-1 o.midValues(end)+1])
set(gca,'YScale','log');
grid on
maxLim = 10^(ceil(max(log10([o.ratiosDofr.values; o.ratiosPam.values]))/0.5)*0.5);
minLim = 10^(floor(min(log10([o.ratiosDofr.values; o.ratiosPam.values]))/0.5)*0.5);
ylim([minLim maxLim]);
set(gca,'FontSize',fs);
ax = gca;
ax.YLabel.Interpreter = 'latex';               
ax.YLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
set(gca,'Box','on');
% legh = legend([b1,b2],'DOFR','PAM','Location','SouthWest','NumColumns',2);
% legh.ItemTokenSize(1) = 15;
% legh.BoxFace.ColorType = 'truecoloralpha';
% legh.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
title('(d) Short events (14 s)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
ax.YTick = [1e-2 1e-1 1e0 1e1 1e2 1e3 1e4 1e5];
ax.YTickLabel = {'10^{-2}','','10^{0}','','10^{2}','','10^{4}',''};
ax.MinorGridLineStyle = 'none';

%%
if(isExport)
    % Not possible to save as vector graphics due to large number of
    % outlier points.
    exportgraphics(gcf,'./exportedFigures/fig9.png','Resolution',1000);
end
clear isExport

%% Print some of the values:
ix = find(strcmp(selectedMethods,'OFRCO_2'));
s_ix = find(strcmp(selectedMethods,'standard'));

disp('400 s subcycles, OFR CO_2 method:');
disp(['DOFR: Median: ' num2str(round(d.long.median(ix),2)),...
    ', P01: ' num2str(round(d.long.p01(ix),2)),...
    ', P99: ' num2str(round(d.long.p99(ix),2)),...
    ', P25: ' num2str(round(d.long.p25(ix),2)),...
    ', P75: ' num2str(round(d.long.p75(ix),2))]);
disp(['PAM: Median: ' num2str(round(p.long.median(ix),2)),...
    ', P01: ' num2str(round(p.long.p01(ix),2)),...
    ', P99: ' num2str(round(p.long.p99(ix),2)),...
    ', P25: ' num2str(round(p.long.p25(ix),2)),...
    ', P75: ' num2str(round(p.long.p75(ix),2))]);
disp('--')
disp('14 s events, standard method:');
disp(['DOFR: Median: ' num2str(round(d.short.median(s_ix),2)),...
    ', P01: ' num2str(round(d.short.p01(s_ix),2)),...
    ', P99: ' num2str(round(d.short.p99(s_ix),2)),...
    ', P25: ' num2str(round(d.short.p25(s_ix),2)),...
    ', P75: ' num2str(round(d.short.p75(s_ix),2))]);
disp(['PAM: Median: ' num2str(round(p.short.median(s_ix),2)),...
    ', P01: ' num2str(round(p.short.p01(s_ix),2)),...
    ', P99: ' num2str(round(p.short.p99(s_ix),2)),...
    ', P25: ' num2str(round(p.short.p25(s_ix),2)),...
    ', P75: ' num2str(round(p.short.p75(s_ix),2))]);
disp('14 s events, OFR_CO2 method:');
disp(['DOFR: Median: ' num2str(round(d.short.median(ix),2)),...
    ', P01: ' num2str(round(d.short.p01(ix),2)),...
    ', P99: ' num2str(round(d.short.p99(ix),2)),...
    ', P25: ' num2str(round(d.short.p25(ix),2)),...
    ', P75: ' num2str(round(d.short.p75(ix),2))]);
disp(['PAM: Median: ' num2str(round(p.short.median(ix),2)),...
    ', P01: ' num2str(round(p.short.p01(ix),2)),...
    ', P99: ' num2str(round(p.short.p99(ix),2)),...
    ', P25: ' num2str(round(p.short.p25(ix),2)),...
    ', P75: ' num2str(round(p.short.p75(ix),2))]);

%% Clear workspace
clear ax b1 b2 binWidth d edges fh file2load fs heightCm isSave ix k;
clear legh legh3 maxLim maxRatio maxVal methodLabels1 methodLabels2;
clear methods midpoints minLim minRatio minVal nDofr nPam numBins o p;
clear rotation s_ix selectedMethods t widthCm isConverted;

