%   Pauli Simonen, Tampere University, 2023
%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
widthCm = 16; % Figure width (cm)
heightCm = 9; % Figure height (cm)

method = 'standard';
binWidth = 10;

%% Load data and initialize reactors
load ./data/hotNedc.mat hotNedc
load ./data/coldNedc.mat coldNedc

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

hot = Cycle('Hot NEDC', hotNedc, soaYield, endBufferLength);
cold = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);

% Don't display the warning related to deconvolution:
warning('off','Cycle:deconvolution');

% Add reactors to the cycle:
hot.addOfr(dofr);
hot.addOfr(pam);
cold.addOfr(dofr);
cold.addOfr(pam);

shiftingTimesDofr = 0.2*dofr.t_peak:0.1*dofr.t_peak:2.2*dofr.t_peak;
shiftingTimesPam = 0.2*pam.t_peak:0.1*pam.t_peak:2.2*pam.t_peak;

clear soaYield endBufferLength hotNedc coldNedc;

%% Scan the delay correction coefficient and calculate the ratios of 10 s bin for each coefficient
ratiosDofrHot = table;
ratiosDofrCold = table;
ratiosPamHot = table;
ratiosPamCold = table;
bins = struct;
bins.all.startIx = (1:binWidth:hot.original.cycleLength-binWidth)';
bins.all.endIx = bins.all.startIx+binWidth;

for j=1:numel(shiftingTimesDofr)
    hot.setOfrDelayCorrection('DOFR',shiftingTimesDofr(j));
    hot.setOfrDelayCorrection('PAM',shiftingTimesPam(j));
    cold.setOfrDelayCorrection('DOFR',shiftingTimesDofr(j));
    cold.setOfrDelayCorrection('PAM',shiftingTimesPam(j));

    [ofrsBinnedHot, originalBinnedHot] = handleBinnedData(hot, bins, method);
    [ofrsBinnedCold, originalBinnedCold] = handleBinnedData(cold, bins, method);

    t= table;        
    t.values = ofrsBinnedHot.DOFR.all.pf./originalBinnedHot.all.pf;
    t.x = shiftingTimesDofr(j)./dofr.t_peak.*ones(size(t.values));
    ratiosDofrHot = [ratiosDofrHot; t];

    t = table;        
    t.values = ofrsBinnedCold.DOFR.all.pf./originalBinnedCold.all.pf;
    t.x = shiftingTimesDofr(j)./dofr.t_peak.*ones(size(t.values));
    ratiosDofrCold = [ratiosDofrCold; t];

    t = table;        
    t.values = ofrsBinnedHot.PAM.all.pf./originalBinnedHot.all.pf;
    t.x = shiftingTimesPam(j)./pam.t_peak.*ones(size(t.values));
    ratiosPamHot = [ratiosPamHot; t];

    t = table;        
    t.values = ofrsBinnedCold.PAM.all.pf./originalBinnedCold.all.pf;
    t.x = shiftingTimesPam(j)./pam.t_peak.*ones(size(t.values));
    ratiosPamCold = [ratiosPamCold; t];
end


%% Plot figure
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Color = 'w';
fh.Renderer = 'painters';

t = tiledlayout(2,2);
t.Padding = 'none';
t.TileSpacing = 'compact';

% Panel 1
nexttile;
b = boxchart_ps(ratiosDofrCold,'blue',0.2);
b.BoxWidth = 0.05;
xlim([0.1 2.3]);
set(gca,'YScale','log')
ylim([0.1 10]);
grid on;
title('DOFR; Cold-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
ax = gca;
ax.YLabel.Interpreter = 'latex';               
ax.YLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
set(gca,'FontSize',fs);

% Panel 2
nexttile;
b = boxchart_ps(ratiosPamCold,'blue',0.2);
b.BoxWidth = 0.05;
xlim([0.1 2.3]);
set(gca,'YScale','log')
ylim([0.1 10]);
grid on;
title('PAM; Cold-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
set(gca,'FontSize',fs);

% Panel 3
nexttile;
b = boxchart_ps(ratiosDofrHot,'blue',0.2);
b.BoxWidth = 0.05;
xlim([0.1 2.3]);
set(gca,'YScale','log')
ylim([0.1 10]);
grid on;
title('DOFR; Hot-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
xlabel('\tau_r / \tau_{peak}');
ax = gca;
ax.YLabel.Interpreter = 'latex';               
ax.YLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
set(gca,'FontSize',fs);

% Panel 4
nexttile;
b = boxchart_ps(ratiosPamHot,'blue',0.2);
b.BoxWidth = 0.05;
xlim([0.1 2.3]);
set(gca,'YScale','log')
ylim([0.1 10]);
grid on;
title('PAM; Hot-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
xlabel('\tau_r / \tau_{peak}');
set(gca,'FontSize',fs);

%% Export
if(isExport)
    exportgraphics(gcf,'./exportedFigures/figS10.pdf');
end

%% Re-enable deconvolution warning:
warning('on','Cycle:deconvolution');

%% Clear workspace
clear ax b bins binWidth cold dofr fh fs heightCm hot isExport j lw;
clear method ofrsBinnedCold ofrsBinnedHot originalBinnedCold;
clear originalBinnedHot pam ratiosDofrCold ratiosDofrHot;
clear ratiosPamCold ratiosPamHot shiftingTimesDofr shiftingTimesPam;
clear t widthCm;
