%   Pauli Simonen, Tampere University, 2023
%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
widthCm = 16; % Figure width (cm)
heightCm = 9; % Figure height (cm)

method = 'standard';

%% Load data and initialize reactors
load ./data/hotNedc.mat hotNedc
load ./data/coldNedc.mat coldNedc

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

hot = Cycle('Hot NEDC', hotNedc, soaYield, endBufferLength);
cold = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);

% Add reactors to the cycle:
hot.addOfr(dofr);
hot.addOfr(pam);
cold.addOfr(dofr);
cold.addOfr(pam);

clear soaYield endBufferLength hotNedc coldNedc;


%% Scan the bin width and calculate the PF ratios for each bin width
ratiosDofrHot = table;
ratiosDofrCold = table;
ratiosPamHot = table;
ratiosPamCold = table;

binWidths =  [1, 10, (30:20:150)];

for k=1:numel(binWidths)
    bins = struct;
    bins.all.startIx = (1:binWidths(k):hot.original.cycleLength-binWidths(k))';
    bins.all.endIx = bins.all.startIx+binWidths(k);

    [ofrsBinnedHot, originalBinnedHot] = handleBinnedData(hot, bins, method);
    [ofrsBinnedCold, originalBinnedCold] = handleBinnedData(cold, bins, method);
    
    t= table;        
    t.values = ofrsBinnedHot.DOFR.all.pf./originalBinnedHot.all.pf;
    t.x = binWidths(k).*ones(size(t.values));
    ratiosDofrHot = [ratiosDofrHot; t];

    t = table;        
    t.values = ofrsBinnedCold.DOFR.all.pf./originalBinnedCold.all.pf;
    t.x = binWidths(k).*ones(size(t.values));
    ratiosDofrCold = [ratiosDofrCold; t];

    t = table;        
    t.values = ofrsBinnedHot.PAM.all.pf./originalBinnedHot.all.pf;
    t.x = binWidths(k).*ones(size(t.values));
    ratiosPamHot = [ratiosPamHot; t];

    t = table;        
    t.values = ofrsBinnedCold.PAM.all.pf./originalBinnedCold.all.pf;
    t.x = binWidths(k).*ones(size(t.values));
    ratiosPamCold = [ratiosPamCold; t];

end

%% Plot figure
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Color = 'w';
fh.Renderer = 'painters';

t = tiledlayout(2,2);
t.Padding = 'none';
t.TileSpacing = 'compact';

ax = {};


% Panel 1
nexttile;
b = boxchart_ps(ratiosDofrCold,'blue',2);
b.BoxWidth = 5;
xlim([-10 160]);
set(gca,'XTick',binWidths);
set(gca,'YScale','log');
grid on;
title('DOFR; Cold-start NEDC');
set(gca,'FontSize',fs);
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
ax{1} = gca;
ax{1}.YLabel.Interpreter = 'latex';               
ax{1}.YLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
ylims = ylim;
yMin = ylims(1);
yMax = ylims(2);
minY = yMin;
maxY = yMax;

% Panel 2
nexttile;
b = boxchart_ps(ratiosPamCold,'blue',2);
b.BoxWidth = 5;
xlim([-10 160]);
set(gca,'XTick',binWidths);
set(gca,'YScale','log');
grid on;
title('PAM; Cold-start NEDC');
set(gca,'FontSize',fs);
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
ax{2} = gca;
ylims = ylim;
yMin = ylims(1);
yMax = ylims(2);
if(yMin < minY)
    minY = yMin;
end
if(yMax > maxY)
    maxY = yMax;
end

% Panel 3
nexttile;
b = boxchart_ps(ratiosDofrHot,'blue',2);
b.BoxWidth = 5;
xlim([-10 160]);
set(gca,'XTick',binWidths);
set(gca,'YScale','log');
grid on;
title('DOFR; Hot-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
xlabel('Bin width (s)');
ax{3} = gca;
ax{3}.YLabel.Interpreter = 'latex';               
ax{3}.YLabel.String = '$\frac{\rm Reactor\ PF}{\rm Reference\ PF}$';
ylims = ylim;
yMin = ylims(1);
yMax = ylims(2);
if(yMin < minY)
    minY = yMin;
end
if(yMax > maxY)
    maxY = yMax;
end

% Panel 4
nexttile;
b = boxchart_ps(ratiosPamHot,'blue',2);
b.BoxWidth = 5;
xlim([-10 160]);
set(gca,'XTick',binWidths);
set(gca,'YScale','log');
grid on;
title('PAM; Hot-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);
xlabel('Bin width (s)');
ax{4} = gca;
ylims = ylim;
yMin = ylims(1);
yMax = ylims(2);
if(yMin < minY)
    minY = yMin;
end
if(yMax > maxY)
    maxY = yMax;
end

for k=1:4
    set(ax{k},'YLim',[minY maxY]);
end

%% Export pdf:
if(isExport)
    exportgraphics(gcf,'./exportedFigures/figS11.pdf');
end
clear isExport

% %% Clear workspace
% clear ax b bins binWidths cold dofr fh fs heightCm hot k lw maxY method;
% clear minY ofrsBinnedCold ofrsBinnedHot originalBinnedCold originalBinnedHot;
% clear pam ratiosDofrCold ratiosDofrHot ratiosPamCold ratiosPamHot t;
% clear widthCm ylims yMax yMin;