%   Pauli Simonen, Tampere University, 2023
%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
lw = 1.5; % Line width
widthCm = 8.3; % Figure width (cm)
heightCm = 6; % Figure height (cm)

method = 'CVS_noDelayCorrection';

%% Load data and initialize reactors
load ./data/coldNedc.mat coldNedc

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

cycle = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);

% Add reactors to the cycle:
cycle.addOfr(dofr);
cycle.addOfr(pam);

clear soaYield endBufferLength coldNedc;

%% Resolve OFR emission index
emissionIx = NaN;

for k=1:numel(cycle.(cycle.ofrNames{1}).emissions)
    if(strcmp(cycle.(cycle.ofrNames{1}).emissions{k}.method, method))
        emissionIx = k;
        break;
    end
end
if(isnan(emissionIx))
    error(['Calculation method ''' method ''' not found.']);
end

clear k method

%% Plot
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Renderer = 'painters';
fh.Color = 'w';

% Colors:
blueC = [0 114 189]./255;
redC = [217 83 25]./255;

plot(cycle.DOFR.time-cycle.original.cycleLength,cycle.DOFR.emissions{emissionIx}.cumulativePf_mg_per_kgfuel./cycle.original.emissions.cumulativePf_mg_per_kgfuel,'LineWidth',1.5,'Color',blueC);
hold on
plot(cycle.PAM.time-cycle.original.cycleLength,cycle.PAM.emissions{emissionIx}.cumulativePf_mg_per_kgfuel./cycle.original.emissions.cumulativePf_mg_per_kgfuel,'LineWidth',1.5,'Color',redC,'LineStyle',':');
line([dofr.t_mean dofr.t_mean],[0 2],'LineStyle','--','LineWidth',1.5,'Color',blueC);
line([pam.t_mean pam.t_mean],[0 2],'LineStyle','--','LineWidth',1.5,'Color',redC);
xlim([-50 300]);
ylim([0.9 1.35]);
grid on;
legh = legend('DOFR','PAM','\tau_{mean, DOFR}','\tau_{mean, PAM}','Location','NorthEast');
legh.ItemTokenSize(1) = 15;
legh.BoxFace.ColorType = 'truecoloralpha';
legh.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
ylabel('OFR Total PF / Reference Total PF');
xlabel('Time (s)');
set(gca,'FontSize',fs);

if(isExport)
    exportgraphics(gcf,'./exportedFigures/figS13.pdf');
end
clear isExport;

%% Clear workspace
clear blueC redC cycle dofr emissionIx fh fs heightCm legh lw pam widthCm;

