%   Pauli Simonen, Tampere University, 2023
% Same as Fig 4, but plot correlations instead of histograms
%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
lw = 1.5; % Line width
lw_marker = 1.0; % Marker line width
widthCm = 16; % Figure width (cm)
heightCm = 9; % Figure height (cm)

method = 'standard'; % The method to use for calculating OFR emissions

%% Load data and initialize reactors
load ./data/hotNedc.mat hotNedc
load ./data/coldNedc.mat coldNedc

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

hot = Cycle('Hot NEDC', hotNedc, soaYield, endBufferLength);
cold = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);

% Add reactors to the cycle:
hot.addOfr(dofr);
hot.addOfr(pam);
cold.addOfr(dofr);
cold.addOfr(pam);

clear dofr pam soaYield endBufferLength hotNedc coldNedc;

%% Divide NEDC cycle into bins
% Load theoretical speed and adjust time to match recorded speed:
load ./data/nedcSpeed.mat nedcSpeed
nedcSpeed.time = nedcSpeed.time + 1;

bins = struct;
[bins.accelerations, bins.idles, bins.constSpeeds, bins.decelerations] =...
    binCycleData(nedcSpeed.time, nedcSpeed.speed);

[ofrsBinnedCold, originalBinnedCold] = handleBinnedData(cold, bins, method);
[ofrsBinnedHot, originalBinnedHot] = handleBinnedData(hot, bins, method);

%% Plot figure
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Color = 'w';

t = tiledlayout(2,4);
t.Padding = 'none';
t.TileSpacing = 'compact';

reactors = fields(ofrsBinnedCold);
cases = {'accelerations','constSpeeds','idles','decelerations'};
caseTitles = {'Acceleration','Constant speed','Idle','Deceleration'};
cycles = {'coldNedc','hotNedc'};
cycleTitles = {'CS','HS'};
figLabels = {'(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'};

markers = {'o','s'};
blueC = [0 114 189]./255;
redC = [217 83 25]./255;

markerColors = {blueC, redC};

ofrs = struct;
ofrs.coldNedc = ofrsBinnedCold;
ofrs.hotNedc = ofrsBinnedHot;
drivingCycles.coldNedc = originalBinnedCold;
drivingCycles.hotNedc = originalBinnedHot;


clear tileh
ii = 1;
clear legh
jj = 0;
for c=1:numel(cycles)
    maxlims = zeros(numel(cases),1);
    minlims = zeros(numel(cases),1);
    for j=1:numel(cases)
        jj = jj + 1;
        tileh(jj) = nexttile;
        legStr = {};
        maxlim = 0;
        minlim = 1e4;
        maxValLin = 0;
        for k=1:numel(reactors)
            plot(drivingCycles.(cycles{c}).(cases{j}).pf,ofrs.(cycles{c}).(reactors{k}).(cases{j}).pf,'LineWidth',lw_marker,'MarkerSize',4,'Color',markerColors{k},'Marker',markers{k},'LineStyle','none');
            maxlimTemp = max(ceil(log10([drivingCycles.(cycles{c}).(cases{j}).pf; ofrs.(cycles{c}).(reactors{k}).(cases{j}).pf])));
            maxlimTemp = 10^maxlimTemp;
            minlimTemp = min(floor(log10([drivingCycles.(cycles{c}).(cases{j}).pf; ofrs.(cycles{c}).(reactors{k}).(cases{j}).pf])));
            minlimTemp = 10^minlimTemp;
            maxlim = max([maxlim maxlimTemp]);
            minlim = min([minlim minlimTemp]);
            maxValLin = max([maxValLin; drivingCycles.(cycles{c}).(cases{j}).pf; ofrs.(cycles{c}).(reactors{k}).(cases{j}).pf]);
            legStr{end+1} = reactors{k};
            hold on;
        end

        grid on;
        title([figLabels{ii} ' ' cycleTitles{c} ', ' caseTitles{j}]);
        set(gca,'TitleFontWeight','normal');
        set(gca,'TitleFontSizeMultiplier',1.0);
        if(ii >=5)
            xlabel('Reference PF (mg kg_{fuel}^{-1})');
        end
        if(mod(ii-1,4)==0)
            ylabel('OFR PF (mg kg_{fuel}^{-1})');
        end
        set(gca,'FontSize',fs);
        % set(gca,'LooseInset',[0 0 0 0]);
        if(contains(cycles{c},'Cold','IgnoreCase',true))
            set(gca,'xscale','log');
            set(gca,'yscale','log');
            ylim([minlim maxlim]);
            xlim([minlim maxlim]);
        else
            ylims = ylim;
            xlims = xlim;
            maxlim = max([ylims xlims]);
            minlim = min([ylims xlims]);
            if(maxValLin/maxlim > 0.98)
                maxlim = maxlim + 10;
            end
            ylim([minlim maxlim]);
            xlim([minlim maxlim]);
        end


        line([minlim maxlim],[minlim maxlim],'Color','k','LineStyle','--','LineWidth',lw);
        legStr{end+1} = '1:1';
        if(jj <= 4)
            legh(jj) = legend(legStr,'Location','SouthEast');
        else
            legh(jj) = legend(legStr,'Location','NorthWest');
        end
        legh(jj).ItemTokenSize(1) = 15;
        legh(jj).BoxFace.ColorType = 'truecoloralpha';
        legh(jj).BoxFace.ColorData=uint8(255*[1 1 1 0.75]');

        set(gca,'XTick',get(gca,'YTick'));

        ii = ii + 1;
    end
    maxlim = max(maxlims);
    minlim = min(minlims);
end

% Adjust legend positions:
for k=1:jj
    if(k <= 4)
        legh(k).Position(1) = legh(k).Position(1) + 0.0145;
        legh(k).Position(2) = legh(k).Position(2) - 0.025;
    else
        legh(k).Position(1) = legh(k).Position(1) - 0.0145;
        legh(k).Position(2) = legh(k).Position(2) + 0.025;
    end
end

%% Export pdf:
if(isExport)
    exportgraphics(gcf,'./exportedFigures/figS15.pdf');
end
clear isExport

%% Clear workspace
clear bins blueC redC c cases caseTitles cold cycles cycleTitles;
clear drivingCycles fh figLabels fs heightCm hot ii j jj k legh legStr;
clear lw lw_marker markerColors markers maxlim maxlims maxlimTemp;
clear maxValLin method minlim minlims minlimTemp nedcSpeed ofrs;
clear ofrsBinnedCold ofrsBinnedHot originalBinnedCold originalBinnedHot;
clear reactors t tileh widthCm xlims ylims;