%   Pauli Simonen, Tampere University, 2023
%% 
isSave = true;
isExport = true;
endBufferLength = 600;
soaYield = 0.15;

fs = 8;
lwT = 1.7; % Line width for True SOA and PAM SOA
lw = 1.2; % Line width
widthCm = 16;
heightCm = 12;

blueC = [0 114 189]./255;
redC = [217 83 25]./255;
grayC = [0.3 0.3 0.3];

%% Deconvolve data or load deconvolved data
% Cold-start cycle:
file2load = './data/deconvolutions/dc_2023_10_31_20_00_coldNedc.mat';
if(~isfile(file2load))
    disp('Pre-calculated deconvolved data not found - deconvolving now...');
    maxIterations = 5e8;

    load ./data/coldNedc.mat coldNedc
    
    dofr = Ofr('DOFR');
    dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.
    
    pam = Ofr('PAM');
    pam.setRtd(calcPamRtd(700));
    
    cold = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);
    
    % Add reactors to the cycle:
    cold.addOfr(dofr);
    cold.addOfr(pam);
    
    clear dofr pam coldNedc;
    
    cold.deconvolveOfrData('DOFR',150,maxIterations);
    cold.deconvolveOfrData('PAM',500,maxIterations);

    if(isSave)
        fnam = ['dc_' datestr(now(), 'yyyy_mm_dd_HH_MM')];
        save(['./data/deconvolutions/' fnam '_coldNedc.mat'],'cold');
        disp(['File ./data/deconvolutions/' fnam '_coldNedc.mat saved.']);
    end
    clear maxIterations
else
    disp('Loading the deconvolved data...');
    load(file2load);
    disp('Deconvolved data loaded.');

    clear file2load
end

% Hot-start cycle:
file2load = './data/deconvolutions/dc_2023_10_31_21_04_hotNedc.mat';
if(~isfile(file2load))
    disp('Pre-calculated deconvolved data not found - deconvolving now...');
    maxIterations = 5e8;

    load ./data/hotNedc.mat hotNedc
    
    dofr = Ofr('DOFR');
    dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.
    
    pam = Ofr('PAM');
    pam.setRtd(calcPamRtd(700));
    
    hot = Cycle('Hot NEDC', hotNedc, soaYield, endBufferLength);
    
    % Add reactors to the cycle:
    hot.addOfr(dofr);
    hot.addOfr(pam);
    
    clear dofr pam soaYield endBufferLength hotNedc;
    
    hot.deconvolveOfrData('DOFR',150,maxIterations);
    hot.deconvolveOfrData('PAM',500,maxIterations);

    if(isSave)
        fnam = ['dc_' datestr(now(), 'yyyy_mm_dd_HH_MM')];
        save(['./data/deconvolutions/' fnam '_hotNedc.mat'],'hot');
        disp(['File ./data/deconvolutions/' fnam '_hotNedc.mat saved.']);
    end
    clear maxIterations
else
    disp('Loading the deconvolved data...');
    load(file2load);
    disp('Deconvolved data loaded.');
    
    clear file2load
end

clear endBufferLength soaYield isSave

%% Plot
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.Renderer = 'painters';
fh.Color = 'w';

t = tiledlayout(2,1);
t.Padding = 'none';
t.TileSpacing = 'compact';

nexttile;
plot(cold.original.tailpipe.soa_g_per_m3.*1e3,'LineWidth',lwT,'Color',grayC);
hold on
plot(cold.DOFR.tailpipeDeconvoluted.soa_g_per_m3.*1e3,'LineWidth',lw,'Color',blueC);
plot(cold.PAM.tailpipeDeconvoluted.soa_g_per_m3.*1e3,'LineWidth',lwT,'Color',redC,'LineStyle',':');
xlim([0 1200]);
grid on;
set(gca,'FontSize',fs);
ylabel('SOA (mg m^{-3})')
legend('Reference','DOFR','PAM','Location','NorthEast');
title('(a) Cold-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

nexttile;
plot(hot.original.tailpipe.soa_g_per_m3.*1e3,'LineWidth',lwT,'Color',grayC);
hold on
plot(hot.DOFR.tailpipeDeconvoluted.soa_g_per_m3.*1e3,'LineWidth',lw,'Color',blueC,'LineStyle','-');
plot(hot.PAM.tailpipeDeconvoluted.soa_g_per_m3.*1e3,'LineWidth',lwT,'Color',redC,'LineStyle',':');
xlim([0 1200]);
grid on;
set(gca,'FontSize',fs);
ylabel('SOA (mg m^{-3})')
xlabel('Time (s)');
legend('Reference','DOFR','PAM','Location','NorthEast');
title('(b) Hot-start NEDC');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

%% Export
if(isExport)
    exportgraphics(gcf,'./exportedFigures/figS16.pdf');
end
clear isExport

%% Clear
clear blueC redC grayC cold hot fh fs lw lwT t widthCm heightCm
