%   Pauli Simonen, Tampere University, 2023
% Same as Fig. 2 but for the cold-start cycle

%% Constants
isExport = true; % Shall the figure be saved as pdf?
endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
lw = 1.5; % Line width
widthCm = 16; % Figure width (cm)
heightCm = 9.5; % Figure height (cm)

co2Method = 'OFR CO_2'; % The method to use for calculating OFR CO2 emissions
soaMethod = 'standard'; % The method to use for calculating OFR SOA emissions

%% Load data and initialize reactors
load ./data/coldNedc.mat coldNedc

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

cycle = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);

% Add reactors to the cycle:
cycle.addOfr(dofr);
cycle.addOfr(pam);

clear dofr pam soaYield endBufferLength coldNedc;

%% Resolve OFR emission indices
co2EmissionIx = NaN;
soaEmissionIx = NaN;

for k=1:numel(cycle.(cycle.ofrNames{1}).emissions)
    if(strcmp(cycle.(cycle.ofrNames{1}).emissions{k}.method, co2Method))
        co2EmissionIx = k;
    end
    if(strcmp(cycle.(cycle.ofrNames{1}).emissions{k}.method, soaMethod))
        soaEmissionIx = k;
    end
    if(~isnan(co2EmissionIx) && ~isnan(soaEmissionIx))
        break;
    end
end
if(isnan(co2EmissionIx))
    error(['Calculation method ''' co2Method ''' not found.']);
end
if(isnan(soaEmissionIx))
    error(['Calculation method ''' soaMethod ''' not found.']);
end

clear k co2Method soaMethod

%% Create figure
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.PaperPositionMode = 'auto';
fh.Renderer = 'painters';
fh.Color = 'w';

% Colors:
blueC = [0 114 189]./255;
redC = [217 83 25]./255;
grayC = [0.3 0.3 0.3];

t = tiledlayout(2,3);
t.Padding = 'none';
t.TileSpacing = 'compact';

clear widthCm heightCm

%% Plot panel (a)
% CO2, Q_exh, Speed
nexttile(1,[1 2]);
legStr = {};

% Need to plot speed on left Y-axis because of Matlab issues...
maxValue = max(cycle.original.tailpipe.co2_percent);
sh = plotSpeedPatch(cycle.original.time,cycle.original.speed./...
    max(cycle.original.speed).*maxValue,[200 200 200]./255);
clear sh;
legStr{end+1} = 'Speed';
hold on;

set(gca,'YColor','k');
plot(cycle.original.time,cycle.original.exhFlow_m3_per_s.*60,'LineWidth',lw,'Color',redC);
hold on;
plot(cycle.original.time,cycle.original.tailpipe.co2_percent,'LineWidth',lw,'Color',grayC);
legStr{end+1} = 'Q_{exh}';
legStr{end+1} = '[CO_2]_{True}';
ix = 1;
for ofr=cycle.ofrNames
    if(ix==1)
        plot(cycle.(ofr{1}).time,cycle.(ofr{1}).tailpipe.co2_percent_shifted,'LineWidth',lw,'LineStyle','-','Color',blueC);
    elseif(ix==2)
        plot(cycle.(ofr{1}).time,cycle.(ofr{1}).tailpipe.co2_percent_shifted,'LineWidth',lw,'LineStyle',':','Color',redC);
    else
        plot(cycle.(ofr{1}).time,cycle.(ofr{1}).tailpipe.co2_percent_shifted,'LineWidth',lw,'LineStyle','-');
    end
    legStr{end+1} = ['[CO_2]_{' ofr{1} '}'];
    ix = ix + 1;
end
clear ix ofr;
ylabel('CO_2 (%), Q_{exh} (m^3 min^{-1})');
set(gca,'FontSize',fs);
grid on;
legh = legend(legStr,'Location','South','NumColumns',2);
legh.ItemTokenSize(1) = 15;
legh.BoxFace.ColorType = 'truecoloralpha';
legh.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');

xlabel('Time (s)');
xlim([0 1300]);
ylims = ylim;

% Adjust the right y axis to show speed correctly:
maxYRight = ylims(2)/maxValue*max(cycle.original.speed);
yyaxis right;
set(gca,'YColor','k');
ylim([0 maxYRight]);
ylabel('Speed (km h^{-1})');
set(gca,'FontSize',fs);

title('(a)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

clear ylims legStr legh maxValue maxYRight

%% Plot panel (b)
% SOA
nexttile(4,[1 2]);
legStr = {};
% Need to plot speed on left Y-axis because of Matlab issues...
maxValue = max(cycle.original.tailpipe.soa_g_per_m3.*1e3); % mg/m3
sh = plotSpeedPatch(cycle.original.time,cycle.original.speed./...
    max(cycle.original.speed).*maxValue,[200 200 200]./255);
hold on;
legStr{end+1} = 'Speed';

plot(cycle.original.time,cycle.original.tailpipe.soa_g_per_m3.*1e3,... % mg/m3
    'LineWidth',lw,'Color',grayC);
hold on;
legStr{end+1} = 'Reference';
ix = 1;
for ofr=cycle.ofrNames
    if(ix==1)
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).tailpipe.soa_g_per_m3_shifted.*1e3,...
            'LineWidth',lw,'LineStyle','-','Color',blueC);
    elseif(ix==2)
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).tailpipe.soa_g_per_m3_shifted.*1e3,...
            'LineWidth',lw,'LineStyle',':','Color',redC);
    else
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).tailpipe.soa_g_per_m3_shifted.*1e3,...
            'LineWidth',lw,'LineStyle','-');
    end
    legStr{end+1} = ofr{1};
    ix = ix+1;
end
ylabel('SOA (mg m^{-3})');
set(gca,'FontSize',fs);
grid on;

legh = legend(legStr,'Location','North');
legh.ItemTokenSize(1) = 15;
legh.BoxFace.ColorType = 'truecoloralpha';
legh.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
xlabel('Time (s)');
xlim([0 1300]);
ylims = ylim;

% Adjust the right y axis to show speed correctly:
maxYRight = ylims(2)/maxValue*max(cycle.original.speed);
yyaxis right;
set(gca,'YColor','k');
ylim([0 maxYRight]);
ylabel('Speed (km h^{-1})');
set(gca,'FontSize',fs);

yyaxis left;
ylim(ylims);
title('(b)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

clear ylims legStr legh maxValue maxYRight
%% Plot panel (c)
% Cumulative CO2
nexttile(3);
legStr = {};

maxValue = max(cycle.original.emissions.cumulativeCo2_g);
sh = plotSpeedPatch(cycle.original.time,cycle.original.speed./...
    max(cycle.original.speed).*maxValue,[200 200 200]./255);
clear sh;
hold on;
legStr{end+1} = 'Speed';


plot(cycle.original.time,cycle.original.emissions.cumulativeCo2_g,...
    'LineWidth',lw,'Color',grayC);
legStr{end+1} = 'Reference';
hold on

ix = 1;
for ofr=cycle.ofrNames
    if(ix==1)
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{co2EmissionIx}.cumulativeCo2_g,...
            'LineWidth',lw,'LineStyle','-','Color',blueC);
    elseif(ix==2)
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{co2EmissionIx}.cumulativeCo2_g,...
            'LineWidth',lw,'LineStyle',':','Color',redC);
    else
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{co2EmissionIx}.cumulativeCo2_g,...
            'LineWidth',lw,'LineStyle','-');
    end
    legStr{end+1} = ofr{1};
    ix = ix+1;
end
clear ofr ix;

ylabel('Emitted CO_2 (g)');
set(gca,'FontSize',fs);
grid on;

legh = legend(legStr,'Location','best');
legh.ItemTokenSize(1) = 15;
legh.BoxFace.ColorType = 'truecoloralpha';
legh.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
xlabel('Time (s)');
xlim([0 1300]);
ylims = ylim;

maxYRight = ylims(2)/maxValue*max(cycle.original.speed);
yyaxis right;
set(gca,'YColor', 'k');
ylim([0 maxYRight]);
ylabel('Speed (km h^{-1})');
set(gca,'FontSize',fs);
yyaxis left;
ylim(ylims);
title('(c)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

clear ylims legStr legh maxValue maxYRight co2EmissionIx

%% Plot panel (d)
% Cumulative SOA
nexttile(6);
legStr = {};

maxValue = max(cycle.original.emissions.cumulativeSoa_g.*1e3); % mg
sh = plotSpeedPatch(cycle.original.time,cycle.original.speed./...
    max(cycle.original.speed).*maxValue,[200 200 200]./255);
clear sh;
hold on;
legStr{end+1} = 'Speed';

plot(cycle.original.time,cycle.original.emissions.cumulativeSoa_g.*1e3,... %mg
    'LineWidth',lw,'Color',grayC);
legStr{end+1} = 'Reference';
hold on

ix = 1;
for ofr=cycle.ofrNames
    if(ix==1)
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{soaEmissionIx}.cumulativeSoa_g.*1e3,...
            'LineWidth',lw,'LineStyle','-','Color',blueC);
    elseif(ix==2)
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{soaEmissionIx}.cumulativeSoa_g.*1e3,...
            'LineWidth',lw,'LineStyle',':','Color',redC);
    else
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{soaEmissionIx}.cumulativeSoa_g.*1e3,...
            'LineWidth',lw,'LineStyle','-');
    end
    legStr{end+1} = ofr{1};
    ix = ix + 1;
end
clear ofr ix;

ylabel('Emitted SOA (mg)');
set(gca,'FontSize',fs);
grid on;
legh = legend(legStr,'Location','best');
legh.ItemTokenSize(1) = 15;
legh.BoxFace.ColorType = 'truecoloralpha';
legh.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
xlabel('Time (s)');
xlim([0 1300]);
ylims = ylim;

maxYRight = ylims(2)/maxValue*max(cycle.original.speed);
yyaxis right;
set(gca,'YColor','k');
ylim([0 maxYRight]);
ylabel('Speed (km h^{-1})');
set(gca,'FontSize',fs);
yyaxis left;
ylim(ylims);
title('(d)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

clear ylims legStr legh maxValue maxYRight soaEmissionIx

%% Export pdf:
if(isExport)
    exportgraphics(gcf,'./exportedFigures/figS2.pdf');
end

clear isExport

%% Clear workspace
clear blueC grayC redC t fh fs lw cycle