%   Pauli Simonen, Tampere University, 2023
% Same as Fig. 3 but for the cold-start cycle

%% Constants
isExport = true; % Shall the figure be saved as pdf?
isLog = true; % Panel b logarithmic scale?

endBufferLength = 600; % 600 s of zero air measurement after the cycle ends
soaYield = 0.15;

fs = 8; % Font size
lw = 1.5; % Line width
widthCm = 16; % Figure width (cm)
heightCm = 5; % Figure height (cm)

method_a = 'standard'; % The method to use for calculating OFR emissions for panel (a)
method_b = 'OFR CO_2'; % The method to use for calculating OFR emissions for panel (b)

subCycles = {'CSUDC','HUDC','EUDC','Full cycle'};
subCycleTimes = [1 392;...
                392 788;...
                788 NaN;... % NaN -> PF is calculated between 788 and end
                1 NaN];

%% Load data and initialize reactors
load ./data/coldNedc.mat coldNedc

dofr = Ofr('DOFR');
dofr.setRtd(calcDofrRtd('low',700)); % UV lamp setting: low, length 700 s.

pam = Ofr('PAM');
pam.setRtd(calcPamRtd(700));

cycle = Cycle('Cold NEDC', coldNedc, soaYield, endBufferLength);

% Add reactors to the cycle:
cycle.addOfr(dofr);
cycle.addOfr(pam);

clear dofr pam soaYield endBufferLength coldNedc;

%% Resolve OFR emission index for panel (b)
emissionIx = NaN;

for k=1:numel(cycle.(cycle.ofrNames{1}).emissions)
    if(strcmp(cycle.(cycle.ofrNames{1}).emissions{k}.method, method_b))
        emissionIx = k;
        break;
    end
end
if(isnan(emissionIx))
    error(['Calculation method ''' method_b ''' not found.']);
end

clear k method_b

%% Create figure
fh = figure;
fh.Units = 'centimeters';
fh.Position(3) = widthCm;
fh.Position(4) = heightCm;
fh.PaperPositionMode = 'auto';
fh.Renderer = 'painters';
fh.Color = 'w';

% Colors:
blueC = [0 114 189]./255;
redC = [217 83 25]./255;
grayC = [0.3 0.3 0.3];

t = tiledlayout(1,3);
t.Padding = 'none';
t.TileSpacing = 'compact';

clear widthCm heightCm
%% Panel (a)
% Total SOA PF
nexttile(1);
barData = [];
legStr = {};
for k=1:numel(subCycles)
    barData(k,1) = cycle.soaPfBetween(subCycleTimes(k,1),subCycleTimes(k,2),...
        'original');
end
clear k;
legStr{end+1} = 'Reference';

for ofr=cycle.ofrNames
    ofrData = [];
    for k=1:numel(subCycles)
        ofrData(k,1) = cycle.soaPfBetween(subCycleTimes(k,1),subCycleTimes(k,2),...
            ofr{1},method_a);
    end
    barData = [barData, ofrData];
    legStr{end+1} = ofr{1};
end
clear ofr ofrData k method_a subCycleTimes

bar(barData);
set(gca,'XTickLabel',subCycles);

legh = legend(legStr,'Location','NorthEast');
legh.ItemTokenSize(1) = 15;
legh.BoxFace.ColorType = 'truecoloralpha';
legh.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
ylabel('Total SOA PF (mg kg_{fuel}^{-1})');
set(gca,'YGrid','on');
set(gca,'FontSize',fs);
title('(a)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

clear barData legh legStr subCycles;
%% Panel (b)
% Momentary PF
nexttile(2,[1 2]);
legStr = {};

maxValue = max(cycle.original.emissions.pf_mg_per_kgfuel);
if(isLog)
        minValue = 10^floor(log10(min(cycle.original.emissions.pf_mg_per_kgfuel)));
        d = max(cycle.original.speed)/log10(maxValue/minValue);
        c = log10(minValue)*d;
        speedMod = (cycle.original.speed+c)./d;
        speedLog = 10.^speedMod;
        sh = plotSpeedPatch(cycle.original.time,speedLog,[200 200 200]./255);
else
    sh = plotSpeedPatch(cycle.original.time,cycle.original.speed./...
        max(cycle.original.speed).*maxValue,[200 200 200]./255);
end
clear sh;
legStr{end+1} = 'Speed';
hold on;

plot(cycle.original.time,cycle.original.emissions.pf_mg_per_kgfuel,...
    'LineWidth',lw,'Color',grayC);

legStr{end+1} = 'Reference';
ix = 1;
for ofr=cycle.ofrNames
    if(ix==1)
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{emissionIx}.pf_mg_per_kgfuel,...
            'LineWidth',lw,'LineStyle','-','Color',blueC);
    elseif(ix==2)
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{emissionIx}.pf_mg_per_kgfuel,...
            'LineWidth',lw,'LineStyle',':','Color',redC);
    else
        plot(cycle.(ofr{1}).time,...
            cycle.(ofr{1}).emissions{emissionIx}.pf_mg_per_kgfuel,...
            'LineWidth',lw,'LineStyle','-');
    end
    legStr{end+1} = ofr{1};
    ix = ix + 1;
end
clear ix ofr emissionIx
ylabel('Momentary PF (mg kg_{fuel}^{-1})');
set(gca,'FontSize',fs);
grid on;

if(isLog)
    set(gca,'YScale','log');
    yMaxLeft = 10^(ceil(log10(max(cycle.original.emissions.pf_mg_per_kgfuel))));
    ylim([minValue yMaxLeft]);
    maxYRight = d*log10(yMaxLeft)-c;
    set(gca,'YTick',10.^(log10(minValue):1:log10(yMaxLeft)));
    legh = legend(legStr,'Location','North','NumColumns',2);
else
    yMaxLeft = 10*ceil(max(cycle.original.emissions.pf_mg_per_kgfuel)/10);
    ylim([0 yMaxLeft]);
    maxYRight = yMaxLeft/maxValue*max(cycle.original.speed);
    legh = legend(legStr,'Location','North');
end
legh.ItemTokenSize(1) = 15;
legh.BoxFace.ColorType = 'truecoloralpha';
legh.BoxFace.ColorData=uint8(255*[1 1 1 0.75]');
xlabel('Time (s)');
xlim([0 1300]);

ylims = ylim;

yyaxis right;
set(gca,'YColor','k');
ylim([0 maxYRight]);
ylabel('Speed (km h^{-1})');
set(gca,'FontSize',fs);
yyaxis left;
ylim(ylims);
title('(b)');
set(gca,'TitleFontWeight','normal');
set(gca,'TitleFontSizeMultiplier',1.0);

% For log scale, the dense grid doesn't look good if it's in
% front, so let's put it to background:
if(isLog)
    ax = gca;
    ax_ = struct(ax);
    ax_.XGridHandle.FrontMajorEdge.Layer = 'back';
    ax_.XGridHandle.FrontMinorEdge.Layer = 'back';
    ax_.YGridHandle.FrontMajorEdge.Layer = 'back';
    ax_.YGridHandle.FrontMinorEdge.Layer = 'back';
    clear ax ax_
end


clear legh ylims maxYRight yMaxLeft maxValue legStr;
clear c d minValue speedLog speedMod;
%% Export pdf:
if(isExport)
    exportgraphics(gcf,'./exportedFigures/figS3.pdf');
end

clear isExport

%% Clear workspace
clear blueC grayC redC t fh fs lw cycle isLog;